@@ -83,7 +83,7 @@
 #ifndef __EMX__
 #define MWM_ICONDIR		XINCL_PATH"/X11/bitmaps:"XINCL_PATH"/X11/pixmaps"
 #else
-#define MWM_ICONDIR		"/XFree86/include/X11/bitmaps:/XFree86/include/X11/pixmaps"
+#define MWM_ICONDIR		"/XFree86/include/X11/bitmaps;/XFree86/include/X11/pixmaps"
 char mwm_icondir[255];
 #endif
 #define USAGE \
@@ -209,6 +209,7 @@ typedef struct ComponentInfo {
     Pixmap          top_shadow_pixmap;
 
     GC              normal_GC;
+    GC              grayed_GC;
     GC              top_GC;
     GC              bot_GC;
     XFontStruct    *font;
