@@ -34,6 +34,7 @@
  ***********************************************************************/
 
 #include "mwm.h"
+#include "X11/bitmaps/stipple"
 
 extern Boolean debugging;
 
@@ -78,6 +79,21 @@ create_menu_gcs(ScreenInfo *scr)
     scr->components[MWM_MENU].normal_GC =
 	XCreateGC(dpy, scr->root_win, gcm, &gcv);
 
+    /* FIXME - not a very good grayed out color */
+    scr->components[MWM_MENU].grayed_GC =
+	XCreateGC(dpy, scr->root_win, gcm, &gcv);
+
+    {
+        Pixmap stipple = XCreateBitmapFromData(dpy, scr->root_win, stipple_bits,
+                                     stipple_width, stipple_height);
+        if (stipple) {
+            XSetFillStyle(dpy, scr->components[MWM_MENU].grayed_GC,
+                          FillOpaqueStippled);
+            XSetStipple(dpy, scr->components[MWM_MENU].grayed_GC,
+                        stipple);
+        }
+    }
+
     if (scr->components[MWM_MENU].top_shadow_pixmap != None &&
 	scr->components[MWM_MENU].top_shadow_pixmap != XmUNSPECIFIED_PIXMAP)
     {
@@ -168,6 +184,10 @@ create_feedback_gcs(ScreenInfo *scr)
     scr->components[MWM_FEEDBACK].normal_GC =
 	XCreateGC(dpy, scr->root_win, gcm, &gcv);
 
+    scr->components[MWM_FEEDBACK].grayed_GC =
+	XCreateGC(dpy, scr->root_win, gcm, &gcv);
+    XSetFillStyle(dpy, scr->components[MWM_FEEDBACK].grayed_GC, FillOpaqueStippled);
+
     if (scr->components[MWM_FEEDBACK].top_shadow_pixmap != None &&
 	scr->components[MWM_FEEDBACK].top_shadow_pixmap != XmUNSPECIFIED_PIXMAP)
     {
@@ -253,6 +273,9 @@ create_pager_gcs(ScreenInfo *scr)
     gcv.background = scr->components[MWM_PAGER].background;
     scr->components[MWM_PAGER].normal_GC =
 	XCreateGC(dpy, scr->root_win, gcm, &gcv);
+    scr->components[MWM_PAGER].grayed_GC =
+	XCreateGC(dpy, scr->root_win, gcm, &gcv);
+    XSetFillStyle(dpy, scr->components[MWM_PAGER].grayed_GC, FillOpaqueStippled);
 }
 
 /*
@@ -295,6 +318,9 @@ create_icon_gcs(ScreenInfo *scr)
 
     scr->components[MWM_ICON].normal_GC =
 	XCreateGC(dpy, scr->root_win, gcm, &gcv);
+    scr->components[MWM_ICON].grayed_GC =
+	XCreateGC(dpy, scr->root_win, gcm, &gcv);
+    XSetFillStyle(dpy, scr->components[MWM_ICON].grayed_GC, FillStippled);
 
     if (scr->components[MWM_ICON].top_shadow_pixmap != None &&
 	scr->components[MWM_ICON].top_shadow_pixmap != XmUNSPECIFIED_PIXMAP)
@@ -453,6 +479,9 @@ create_title_gcs(ScreenInfo *scr)
 
     scr->components[MWM_TITLE_A].normal_GC =
 	XCreateGC(dpy, scr->root_win, gcm, &gcv);
+    scr->components[MWM_TITLE_A].grayed_GC =
+	XCreateGC(dpy, scr->root_win, gcm, &gcv);
+    XSetFillStyle(dpy, scr->components[MWM_TITLE_A].grayed_GC, FillOpaqueStippled);
 
     if (scr->components[MWM_TITLE_A].top_shadow_pixmap != None &&
 	scr->components[MWM_TITLE_A].top_shadow_pixmap != XmUNSPECIFIED_PIXMAP)
