@@ -21,7 +21,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Macros.h,v 1.1 1999/08/04 19:49:50 danny Exp $
+ * $Id: Macros.h,v 1.3 2000/01/03 22:33:01 danny Exp $
  */
 
 /*
@@ -30,6 +30,11 @@
 
 #ifndef _Xbae_Macros_h
 #define _Xbae_Macros_h
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
 /*
  * Prototype wrapper
  */
@@ -51,6 +56,12 @@
 #include <Xm/TextP.h>
 #include <Xm/PrimitiveP.h>
 
+#if 0
+/*	for BX */
+#define DRAW_SHADOW(dpy, draw, tgc, bgc, sz, x, y, w, h, type)	\
+    _XmDrawShadows(dpy, draw, tgc, bgc, x, y, w, h, sz, type)
+
+#else
 #if XmVersion <= 1001
 #define DRAW_SHADOW(dpy, draw, tgc, bgc, sz, x, y, w, h, type)	\
     _XmDrawShadow(dpy, draw, tgc, bgc, sz, x, y, w, h)
@@ -63,6 +74,7 @@
     _XmDrawShadows(dpy, draw, tgc, bgc, x, y, w, h, sz, type)
 #endif
 #endif
+#endif
 
 #if XmVersion <= 1001
 #define DRAW_HIGHLIGHT(dpy, draw, tgc, x, y, w, h, sz, type) \
@@ -610,4 +622,8 @@
 #define IN_GRID_ROW_MODE(mw)	(mw->matrix.grid_type & GRID_MODE_ROW)
 #define IN_GRID_COLUMN_MODE(mw)	(mw->matrix.grid_type & GRID_MODE_COLUMN)
 
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
 #endif /* _Xbae_Macros_h */
