@@ -22,7 +22,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: MatrixP.h,v 1.3 1999/08/17 13:19:40 danny Exp $
+ * $Id: MatrixP.h,v 1.5 1999/12/28 00:10:24 danny Exp $
  */
 
 /*
@@ -32,6 +32,10 @@
 #ifndef _Xbae_MatrixP_h
 #define _Xbae_MatrixP_h
 
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
 #if XmVersion <= 1001
 #	include <Xm/XmP.h>
 #else
@@ -381,10 +385,7 @@ typedef struct {
     Widget	top_clip;
     Widget	bottom_clip;
     Widget	current_parent;		/* Current textField parent window   */
-#if 1
-    /* CELL_WIDGETS, but it's a bad idea to conditionally compile a field here */
     Widget	**cell_widgets;		/* array of widgets for cells	     */
-#endif
 
     GC		cell_bottom_shadow_clip_gc; /* GC for clipped bottom shadow  */
     GC          cell_grid_line_gc;
@@ -437,4 +438,8 @@ typedef struct _XbaeMatrixRec {
 #endif
 
 #undef P
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
 #endif /* _Xbae_MatrixP_h */
