@@ -20,7 +20,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Draw.c,v 1.2 1999/08/17 13:19:44 danny Exp $
+ * $Id: Draw.c,v 1.5 1999/12/12 09:54:56 danny Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,10 +40,8 @@
 
 static void xbaeDrawCellString P((XbaeMatrixWidget, int, int, int, int,
 				  String, Pixel, Pixel));
-#if CELL_WIDGETS
 static void xbaeDrawCellWidget P((XbaeMatrixWidget, int, int, int, int,
 				  Widget, Pixel, Pixel));
-#endif
 static void xbaeDrawCellPixmap P((XbaeMatrixWidget, int, int, int, int,
 				  Pixmap, Pixmap, int, int, Pixel,
 				  Pixel, int));
@@ -346,7 +344,7 @@ Pixel bg, fg;
     xbaeDrawCellShadow(mw, win, row, column, x, y, COLUMN_WIDTH(mw, column),
 		       ROW_HEIGHT(mw), False, False, False);
 }
-#if CELL_WIDGETS
+
 /*
  * Draw a user defined widget in the cell
  */
@@ -366,6 +364,8 @@ Pixel bg, fg;
 
     if (!win)
 	return;
+    if (! XtIsManaged(widget))
+	return;
 
     gc = mw->matrix.draw_gc;
     XSetForeground(XtDisplay(mw), gc, bg);
@@ -376,15 +376,12 @@ Pixel bg, fg;
      * Draw the widget in the cell.
      */
     XtMoveWidget(widget,
-		 x + mw->matrix.cell_shadow_thickness +
-		 mw->matrix.cell_highlight_thickness,
-		 y + mw->matrix.cell_shadow_thickness +
-		 mw->matrix.cell_highlight_thickness);
+		 x + mw->matrix.cell_shadow_thickness + mw->matrix.cell_highlight_thickness,
+		 y + mw->matrix.cell_shadow_thickness + mw->matrix.cell_highlight_thickness);
 
     xbaeDrawCellShadow(mw, win, row, column, x, y, COLUMN_WIDTH(mw, column),
 		       ROW_HEIGHT(mw), False, clipped, False);
 }
-#endif
 
 /*
  * Width in pixels of a character in a given font
@@ -909,14 +906,16 @@ Pixel *fg, *bg;
     /*
      * Compute the background and foreground colours of the cell
      */
-    if (mw->matrix.selected_cells && mw->matrix.selected_cells[row][column])
-	if (mw->matrix.reverse_select)
+    if (mw->matrix.selected_cells && mw->matrix.selected_cells[row][column]) {
+	if (mw->matrix.reverse_select) {
 	    if (mw->matrix.colors)
 		*bg = mw->matrix.colors[row][column];
 	    else
 		*bg = mw->manager.foreground;
+        }
 	else
 	    *bg = mw->matrix.selected_background;
+    }
     else if (mw->matrix.cell_background &&
 	     mw->matrix.cell_background[row][column] !=
 	     mw->core.background_pixel)
@@ -929,14 +928,16 @@ Pixel *fg, *bg;
 	    *bg = mw->matrix.even_row_background;
     }
 
-    if (mw->matrix.selected_cells && mw->matrix.selected_cells[row][column])
-	if (mw->matrix.reverse_select)
+    if (mw->matrix.selected_cells && mw->matrix.selected_cells[row][column]) {
+	if (mw->matrix.reverse_select) {
 	    if (mw->matrix.cell_background)
 		*fg = mw->matrix.cell_background[row][column];
 	    else
 		*fg = mw->core.background_pixel;
+        }
 	else
 	    *fg = mw->matrix.selected_foreground;
+    }
     else if (mw->matrix.colors)
 	*fg = mw->matrix.colors[row][column];
     else
@@ -964,13 +965,10 @@ int row, column;
 
     xbaeComputeCellColors(mw, row, column, &fg, &bg);
 
-#if CELL_WIDGETS
     if (mw->matrix.cell_widgets && mw->matrix.cell_widgets[row][column])
 	xbaeDrawCellWidget(mw, row, column, x, y,
 			   mw->matrix.cell_widgets[row][column], bg, fg);
     else
-#endif
-
 	if (!mw->matrix.draw_cell_callback)
 	{
 	    if (row < mw->matrix.rows && column < mw->matrix.columns)
