@@ -1,3 +1,6 @@
+/*
+ *  $Id: ListTree.c,v 1.7 1999/12/06 13:50:41 amai Exp $
+ */
 /*-----------------------------------------------------------------------------
  * ListTree	A list widget that displays a file manager style tree
  *
@@ -42,7 +45,7 @@
 #ifdef DEBUG
 #include <stdlib.h>
 #include <stdarg.h>
-void DBG(int line,char *fcn,char *fmt, ...)
+static void DBG(int line,char *fcn,char *fmt, ...)
 {
   va_list ap;
   
@@ -54,7 +57,7 @@ void DBG(int line,char *fcn,char *fmt, ...)
 #define DARG __LINE__,__FUNCTION__
 #define DBGW(a) fprintf(stderr,"%s:%d %s()   %s\n",__FILE__,__LINE__,__FUNCTION__, a)
 #else
-void DBG()
+static void DBG()
 {
 }
 #define DARG __LINE__,__FUNCTION__
@@ -1298,7 +1301,8 @@ GetItemPix(ListTreeWidget w, ListTreeItem *item)
       Pixmap pixmap = item->closedPixmap;
 
       Window       root;
-      unsigned int pixwidth, pixheight, pixbw, pixdepth, pixx, pixy;
+      unsigned int pixwidth, pixheight, pixbw, pixdepth;
+      int pixx, pixy;
 
       /* If it is not closed and there is a pixmap for it, then use that one
        * instead.
@@ -2116,7 +2120,7 @@ ListTreeReparentChildren(ListTreeWidget w, ListTreeItem * item, ListTreeItem * n
   return 0;
 }
 
-int
+static int
 AlphabetizeItems(const void *item1, const void *item2)
 {
   return strcmp((*((ListTreeItem **) item1))->text,
