@@ -1,3 +1,5 @@
+/* $Id: test1.c,v 1.3 1999/10/26 09:16:32 amai Exp $ */
+
 #include <stdlib.h>
 #include <Host.h>
 #include <Xm/PushB.h>
@@ -42,7 +44,7 @@ main(int argc, char **argv)
 			       argv, FallBack, NULL); 
 
   one = XmCreatePushButton(toplevel, "Push Me", NULL, 0);
-  XtAddCallback(one, XmNactivateCallback, (void *)Activate, (XtPointer)NULL);
+  XtAddCallback(one, XmNactivateCallback, (XtCallbackProc)Activate, (XtPointer)NULL);
   XtManageChild(one);
 
   XtRealizeWidget(toplevel);
