@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/ComboBox.c,v 1.39 1999/08/30 21:47:39 pgw Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/ComboBox.c,v 1.45 1999/12/30 16:46:37 amai Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/ComboBox.c,v 1.39 1999/08/30 21:47:39 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/ComboBox.c,v 1.45 1999/12/30 16:46:37 amai Exp $";
 
 
 #include <LTconfig.h>
@@ -180,6 +180,7 @@ static void ComboBoxFocusMovedCB(Widget w, XtPointer client_data, XtPointer data
 static void _LTTranslateCoords(Widget w, Position x, Position y, Position *rootx, Position *rooty);
 static void _LTShellSmartPlacement(Widget w);
 static void __LTShellSmartPlacement(Widget w, int *ret_x, int *ret_y);
+static void GetValuesHook(Widget w, ArgList args, Cardinal *num_args);
 
 
 #define VALID(w) (w && XtIsManaged(w))
@@ -401,7 +402,7 @@ static XmSyntheticResource syn_resources[] = {
       Offset(arrow_spacing),
       _XmFromHorizontalPixels,
       _XmToHorizontalPixels
-   }
+   },
 
 };
 
@@ -455,7 +456,7 @@ XmComboBoxClassRec xmComboBoxClassRec = {
       SetValues,                              /* set_values */
       NULL,                                   /* set_values_hook */
       XtInheritSetValuesAlmost,               /* set_values_almost */
-      NULL,                                   /* get_values_hook */
+      GetValuesHook,                          /* get_values_hook */
       NULL,                                   /* accept_focus */
       XtVersion,                              /* version */
       NULL,                                   /* callback offsets */
@@ -559,7 +560,7 @@ CBActivate(
 {
    DEBUGOUT(XdbDebug(__FILE__, w, "%s:CBActivate ...\n",
    	__FILE__));
-   #if 0
+#if 0
    FindComboBox();
    CallSelectionCallbacks(); 
    if (CB_Type(w) == XmDROP_DOWN_COMBO_BOX ||
@@ -568,7 +569,7 @@ CBActivate(
    	PopdownList();
    }
    ComboBoxParentProcess();
-   #endif
+#endif
 }
 
 static void 
@@ -602,10 +603,10 @@ CBCancel(
    String *params, 
    Cardinal *num_params)
 {
-   #if 0
+#if 0
    FindComboBox();
    ComboBoxParentProcess();
-   #endif
+#endif
 }
 
 static void 
@@ -615,10 +616,10 @@ CBDisarm(
    String *params, 
    Cardinal *num_params)
 {
-   #if 0
+#if 0
    FindComboBox();
    DrawArrow();
-   #endif
+#endif
 }
 
 static void 
@@ -655,14 +656,14 @@ CBDropDownList(
    /* pop it up */
    _XmPopupSpringLoaded(CB_ListShell(w));
 
-   #if 0
+#if 0
    XtPopup(CB_ListShell(w), XtGrabNone);
-   #endif
+#endif
 
-   #if 0
+#if 0
    PopdownList();
    CBDisarm();
-   #endif
+#endif
 }
 
 static void 
@@ -692,9 +693,9 @@ CBFocusOut(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
    UnhighlightBorder(w);
 
-   #if 0
+#if 0
    CallSelectionCallbacks(); 
-   #endif
+#endif
 }
 
 #if 0
@@ -705,9 +706,9 @@ CBGetSelectedItem(
    String *params, 
    Cardinal *num_params)
 {
-   #if 0
+#if 0
    GetEditBoxValue();
-   #endif
+#endif
 }
 #endif
 
@@ -766,19 +767,19 @@ CBListAction(
 
       XtVaGetValues(CB_List(w), XmNitemCount, &item_cnt,
                                 NULL);
-      #if 0
+#if 0
       /* FIXME: these resources need to be implemented in the list box */
       XtVaGetValues(CB_List(w), XmNselectedPositions, &selected_pos,
                                 XmNselectedPositionCount, &selected_pos_cnt,
                                 NULL);
 
-      #else
+#else
       if (XmListGetSelectedPos(CB_List(w), &selected_pos, &selected_pos_cnt) == FALSE)
       {
          selected_pos_cnt = 0;
       }
 
-      #endif
+#endif
       if(selected_pos_cnt <= 0)
       {
          pos = 1;
@@ -789,11 +790,11 @@ CBListAction(
          if (selected_pos)
          {
             pos = *selected_pos; /* pick first item */
-            #if 0  /* FIXME: XmNselectedPositions is not implemented. */
-            #else
+#if 0  /* FIXME: XmNselectedPositions is not implemented. */
+#else
             /* Free allocates memory by XmListGetSelectedPos for the selected_pos array */
             XtFree((char *)selected_pos);
-            #endif
+#endif
          }
          else
          {
@@ -830,10 +831,10 @@ CBSetSelectedItem(
    String *params, 
    Cardinal *num_params)
 {
-   #if 0
+#if 0
    GetEditBoxValue();
    SetEditBoxValue();
-   #endif
+#endif
 }
 #endif
 
@@ -974,12 +975,12 @@ ComboBoxParentProcess(Widget w)
    /* FIXTHIS: ComboBoxParentProcess needs to be implemented. */
    Unused(w);
 
-   #if 0
+#if 0
    PopdownList();
    CBDisarm();
    GetEditBoxValue();
    CallSelectionCallbacks();
-   #endif
+#endif
 }
 
 /*
@@ -1059,7 +1060,7 @@ PreferredGeo(Widget w, XtWidgetGeometry *reply)
    DEBUGOUT(XdbDebug(__FILE__, w,
                      "get preferred size of child %s\n",
                      XdbWidgetGeometry2String(reply)));
-};
+}
 
 /*
   FUNCTION: AdjustWidth
@@ -1130,11 +1131,11 @@ ComputeSizeComboBox(Widget w, XtWidgetGeometry *reply)
    reply->width = e_pref.width;
    reply->height = e_pref.height;
 
-   #if 0
+#if 0
    /* start with the size of edit box */
    reply->width = CB_IdealEBWidth(w);
    reply->height = CB_IdealEBHeight(w);
-   #endif
+#endif
 
    /* add the height of scrolled window if type is XmCOMBO_BOX */
    /*
@@ -1174,11 +1175,11 @@ ComputeSizeDropDown(Widget w, XtWidgetGeometry *reply)
    reply->width = e_pref.width;
    reply->height = e_pref.height;
 
-   #if 0
+#if 0
    /* start with the size of edit box */
    reply->width = CB_IdealEBWidth(w);
    reply->height = CB_IdealEBHeight(w);
-   #endif
+#endif
 
    /*
       REMARK:
@@ -1359,22 +1360,22 @@ CreatePulldown(Widget w)
    {
       XtSetArg(args[n], XmNownerEvents, True); n++;
       
-      #if 0
+#if 0
       /* FIXTHIS: GrabModeSync handling is broken in the grab shell. */
       XtSetArg(args[n], XmNgrabStyle, GrabModeSync); n++;
-      #endif
+#endif
 
       /* Despite the function name a GrabShell widget is used */
       CB_ListShell(w) = XmCreateGrabShell(w,CB_LIST_SHELL_CN,args,n);
 
-      #if 1
+#if 1
       /* Why am I doing this? */
 
       /* Add focus handler */
       XmeAddFocusChangeCallback(CB_ListShell(w), 
                                 (XtCallbackProc) ShellFocusMovedCB, 
                                 (XtPointer) w);
-      #endif
+#endif
 
       XtAddCallback(CB_ListShell(w),
                     XmNpopdownCallback, 
@@ -1452,9 +1453,9 @@ CreateScrolledList(Widget w)
                  (XtCallbackProc) ListSelectionCB,
                  (XtPointer) w);
 
-   #if 0
+#if 0
    SetEditBoxValue();
-   #endif
+#endif
 }
 
 /*
@@ -1868,9 +1869,9 @@ GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
        *reply = *request;
    return (XtGeometryResult) XtGeometryYes;
 
-   #if 0
+#if 0
    CheckMinimalSize();
-   #endif
+#endif
 }
 
 /*
@@ -2132,7 +2133,7 @@ PopupEH(Widget w, XtPointer udata, XEvent *xev, Boolean *pb)
    /* Call our action procedure */
    XtCallActionProc(w, "CBArmAndDropDownList", xev, NULL, 0);
    
-   #if 0
+#if 0
    /* Is the grab shell managed? */
    if (!XtIsManaged(CB_ListShell(w)))
    {
@@ -2153,7 +2154,7 @@ PopupEH(Widget w, XtPointer udata, XEvent *xev, Boolean *pb)
       CBDisarm(w,xev,NULL,NULL);
       */
    }
-   #endif
+#endif
 }
 
 /*
@@ -2451,10 +2452,10 @@ SetValues(Widget old,
           Cardinal *num_args)
 {
    Boolean need_refresh = False;
-   #if 0
+#if 0
    XmComboBoxWidget ow = (XmComboBoxWidget) old;
    XmComboBoxWidget nw = (XmComboBoxWidget) c_new;
-   #endif
+#endif
 
    /* FIXTHIS: */
 
@@ -2528,9 +2529,9 @@ ShellPopdownCB(Widget w, XtPointer client_data, XtPointer data)
    Unused(client_data);
    Unused(data);
 
-   #if 0
+#if 0
    CBFocusOut();
-   #endif
+#endif
 } 
 
 static void
@@ -2796,10 +2797,10 @@ AdjustKidsDropDown(Widget w)
    shell_width = edit_width + CB_ArrowSize(w) + CB_ArrowSpacing(w)
                  + (2*(MGR_ShadowThickness(w) + CB_MarginWidth(w)));
 
-   #if 0
+#if 0
    shell_width = XtWidth(w)
                  - (2*CB_HighlightThickness(w));
-   #endif
+#endif
 
    shell_height = XtHeight(CB_ListShell(w));
 
@@ -3213,14 +3214,14 @@ _LTShellSmartPlacement(Widget w)
 {
     int x,y;
 
-    #if 0
+#if 0
     /* Is shell realized? */
     if(!XtIsRealized(w))
     {
         /* Force size calculation */
         XtRealizeWidget(w);
     }
-    #endif
+#endif
 
     __LTShellSmartPlacement(w, &x, &y);
 
@@ -3228,3 +3229,23 @@ _LTShellSmartPlacement(Widget w)
     if(XtX(w) != x || XtY(w) != y) XtVaSetValues(w, XmNx, x, XmNy, y, NULL);
 }
 
+static void
+GetValuesHook(Widget w, ArgList args, Cardinal *num_args)
+{
+	int	i;
+	XrmQuark	tf = XrmStringToQuark(XmNtextField), q,
+			l = XrmStringToQuark(XmNlist);
+	XtPointer	*p;
+
+	for (i=0; i<*num_args; i++) {
+		q = XrmStringToQuark(args[i].name);
+		if (q == tf) {	/* XmNtextField */
+			p = (XtPointer) args[i].value;
+			*p = (XtPointer)XtNameToWidget(w, "Text");
+		} else if (q == l) {	/* XmNlist */
+			p = (XtPointer) args[i].value;
+			*p = (XtPointer)CB_List(w);
+		} else {
+		}
+	}
+}
