@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.9 1999/04/01 00:11:31 jon Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.10 1999/11/19 13:13:16 amai Exp $
  *
  * Copyright (C) 1995-1998 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PanedW.c,v 1.9 1999/04/01 00:11:31 jon Exp $";
+static const char rcsid[] = "$Id: PanedW.c,v 1.10 1999/11/19 13:13:16 amai Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -251,7 +251,7 @@ static Boolean constraint_set_values(Widget current,
 				     ArgList args,
 				     Cardinal *num_args);
 
-Cardinal panedw_insert_position(Widget w);
+static Cardinal panedw_insert_position(Widget w);
 
 /* helpers */
 static void _XmPanedWPreferredSize(Widget w, Widget child,
@@ -999,7 +999,7 @@ realize(Widget w,
 /*
  * Insert panes before sashes and separators.
  */
-Cardinal
+static Cardinal
 panedw_insert_position(Widget w)
 {
     XmPanedWindowWidget pw = (XmPanedWindowWidget)XtParent(w);
