@@ -1,6 +1,6 @@
 /**
  *
- * $Id: BaseClass.c,v 1.11 1999/02/02 18:22:58 gritton Exp $
+ * $Id: BaseClass.c,v 1.12 1999/11/18 19:47:54 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: BaseClass.c,v 1.11 1999/02/02 18:22:58 gritton Exp $";
+static const char rcsid[] = "$Id: BaseClass.c,v 1.12 1999/11/18 19:47:54 amai Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -203,7 +203,7 @@ XmWidgetExtDataStackRec, *XmWidgetExtDataStackPtr;
 /*
  * associate a context with an extension type.
  */
-XContext extContexts[XmDEFAULT_EXTENSION + 1] = { 0 };
+static XContext extContexts[XmDEFAULT_EXTENSION + 1] = { 0 };
 
 static XContext
 FindAssociatedContext(unsigned int extType)
