@@ -1,6 +1,6 @@
 /**
  *
- * $Id: CascadeB.c,v 1.144 1999/07/10 11:52:47 rwscott Exp $
+ * $Id: CascadeB.c,v 1.148 1999/11/19 00:58:34 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: CascadeB.c,v 1.144 1999/07/10 11:52:47 rwscott Exp $";
+static const char rcsid[] = "$Id: CascadeB.c,v 1.148 1999/11/19 00:58:34 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -38,10 +38,10 @@ static const char rcsid[] = "$Id: CascadeB.c,v 1.144 1999/07/10 11:52:47 rwscott
 #include <Xm/MenuUtilP.h>
 #include <Xm/ScreenP.h>
 
-#ifndef __STDC__
-#include <varargs.h>
-#else
+#if STDC_HEADERS
 #include <stdarg.h>
+#else
+#include <varargs.h>
 #endif
 
 #include <XmI/DebugUtil.h>
@@ -423,16 +423,44 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 
     if (Lab_MenuType(new_w) != XmMENU_BAR)
     {
+	int margin;
 
 	if (CB_CascadePixmap(new_w) == XmUNSPECIFIED_PIXMAP &&
 	    CB_Submenu(new_w))
 	    _XmCreateArrowPixmaps(new_w);
 	size_cascade(new_w);
 
-	_LtLabelAccomodateIndicator(new_w, request,
-	    CB_Cascade_width(new_w) + CASCADE_SPACING,
-	    CB_Cascade_height(new_w),
-	    1, Lab_StringDirection(new_w) != XmSTRING_DIRECTION_L_TO_R, False);
+	/* Make sure there's enough room on the side for the cascade pixmap */
+	margin = (CB_Cascade_width(new_w) + CASCADE_SPACING)
+	    - (Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+	       ? Lab_MarginRight(new_w) : Lab_MarginLeft(new_w));
+	if (margin > 0)
+	{
+	    if (Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R)
+		Lab_MarginRight(new_w) += margin;
+	    else
+	    {
+		Lab_MarginLeft(new_w) += margin;
+		Lab_TextRect_x(new_w) += margin;
+	    }
+	    if (!XtWidth(request))
+		XtWidth(new_w) += margin;
+	}
+
+	/* Make sure there's enough room vertically */
+	margin = CB_Cascade_height(new_w) - (Lab_TextRect_height(new_w)
+					     + Lab_MarginTop(new_w)
+					     + Lab_MarginBottom(new_w));
+	if (margin > 0)
+	{
+	    Lab_MarginTop(new_w) += margin >> 1;
+	    Lab_MarginBottom(new_w) += (margin + 1) >> 1;
+	    if (!XtHeight(request))
+	    {
+		Lab_TextRect_y(new_w) += margin >> 1;
+		XtHeight(new_w) += margin;
+	    }
+	}
 
 	place_cascade(new_w);
     }
@@ -458,7 +486,10 @@ resize(Widget w)
     (*superclass->core_class.resize) (w);
 #undef superclass
 
-    place_cascade(w);
+    if (Lab_MenuType(w) != XmMENU_BAR)
+    {
+	place_cascade(w);
+    }
 }
 
 static void
@@ -554,7 +585,10 @@ set_values(Widget old, Widget request, Widget new_w,
 	RC_MenuSubmenu(new_w);
     }
 
-    /* Adjust margins for the cascade pixmap size if necessary */
+    /* Adjust margins for the cascade pixmap size if necessary.
+     * Margins be increased or decreased; if the margin was explicitly set
+     * in this call, don't decrease it past that (though it can get bigger).
+     */
     if (Lab_MenuType(new_w) != XmMENU_BAR &&
 	(CB_Cascade_width(new_w) != CB_Cascade_width(old)
 	 || CB_Cascade_height(new_w) != CB_Cascade_height(old)
@@ -565,10 +599,57 @@ set_values(Widget old, Widget request, Widget new_w,
 	 || Lab_MarginTop(new_w) != Lab_MarginTop(old)
 	 || Lab_MarginBottom(new_w) != Lab_MarginBottom(old)))
     {
-	_LtLabelAccomodateIndicator(new_w, request,
-	    CB_Cascade_width(new_w) + CASCADE_SPACING,
-	    CB_Cascade_height(new_w),
-	    1, Lab_StringDirection(new_w) != XmSTRING_DIRECTION_L_TO_R, True);
+	int margin, tm, bm;
+
+	margin = CB_Cascade_width(new_w) + CASCADE_SPACING
+	    - (Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+	       ? Lab_MarginRight(new_w) : Lab_MarginLeft(new_w));
+	if (margin && (margin > 0 ||
+		       (Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+			? Lab_MarginRight(new_w) == Lab_MarginRight(old)
+			: Lab_MarginLeft(new_w) == Lab_MarginLeft(old))))
+	{
+	    if (Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R)
+		Lab_MarginRight(new_w) += margin;
+	    else
+	    {
+		Lab_MarginLeft(new_w) += margin;
+		Lab_TextRect_x(new_w) += margin;
+	    }
+	    if (Lab_RecomputeSize(new_w) || !XtWidth(request))
+		XtWidth(new_w) += margin;
+	}
+
+	margin = CB_Cascade_height(new_w) - (Lab_TextRect_height(new_w)
+					     + Lab_MarginTop(new_w)
+					     + Lab_MarginBottom(new_w));
+	if (margin)
+	{
+	    tm = margin / 2;
+	    if (tm < (Lab_MarginTop(new_w) == Lab_MarginTop(old)
+		      ? XmDEFAULT_TOP_MARGIN - (int)Lab_MarginTop(new_w)
+		      : 0))
+		tm = (Lab_MarginTop(new_w) == Lab_MarginTop(old)
+		      ? XmDEFAULT_TOP_MARGIN - (int)Lab_MarginTop(new_w)
+		      : 0);
+	    Lab_MarginTop(new_w) += tm;
+	    if (Lab_RecomputeSize(new_w) || !XtHeight(request))
+		XtHeight(new_w) += tm;
+
+	    bm = (margin + 1) / 2;
+	    if (bm < (Lab_MarginBottom(new_w) == Lab_MarginBottom(old)
+		      ? XmDEFAULT_BOTTOM_MARGIN - (int)Lab_MarginBottom(new_w)
+		      : 0))
+		bm = (Lab_MarginBottom(new_w) == Lab_MarginBottom(old)
+		      ? XmDEFAULT_BOTTOM_MARGIN - (int)Lab_MarginBottom(new_w)
+		      : 0);
+	    Lab_MarginBottom(new_w) += bm;
+	    if (Lab_RecomputeSize(new_w) || !XtHeight(request))
+		XtHeight(new_w) += bm;
+
+	    if (tm != bm)
+		Lab_TextRect_y(new_w) += (tm - bm) / 2;
+	}
 
 	refresh_needed = True;
     }
@@ -676,6 +757,25 @@ _XmCascadingPopup(Widget w, XEvent *event, Boolean doCascade)
     /* FIX ME: doCascade? */
     XmAnyCallbackStruct cbs;
 
+#if 1
+    /* rws 29 Sep 1999
+       I really don't like this here, but rowcolumn/test51 shows that it 
+       must be before the CascadingCallback. So I can't roll it into 
+       MenuCascading.
+     */
+    {
+    Widget shell;
+    Widget sub_pane;
+    Boolean was_torn;
+
+	sub_pane = XmIsGadget(w) ?  CBG_Submenu(w) : CB_Submenu(w);
+	if (sub_pane)
+	{
+	    RCClass_MenuProcs(XtClass(sub_pane))(XmMENU_RESTORE_TEAROFF_TO_MENUSHELL,
+				     sub_pane, &shell, &was_torn, event);
+	}
+    }
+#endif
     cbs.reason = XmCR_CASCADING;
     cbs.event = event;
 
@@ -1169,14 +1269,11 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 }
 
 
-#ifdef __STDC__
+#if STDC_HEADERS
 static void
 MenuProcEntry(int proc, Widget w,...)
 {
     va_list arg_list;
-    /*
-    XEvent *event;
-    */
 
     va_start(arg_list, w);
 #else
@@ -1186,9 +1283,6 @@ MenuProcEntry(proc, w, va_alist)
      va_dcl
 {
     va_list arglist;
-    /*
-    XEvent *event;
-    */
 
     va_start(arglist);
 #endif
