@@ -1,6 +1,6 @@
 /**
  *
- * $Id: CascadeBG.c,v 1.128 1999/08/04 16:09:46 rwscott Exp $
+ * $Id: CascadeBG.c,v 1.132 1999/11/19 00:58:35 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: CascadeBG.c,v 1.128 1999/08/04 16:09:46 rwscott Exp $";
+static const char rcsid[] = "$Id: CascadeBG.c,v 1.132 1999/11/19 00:58:35 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -39,10 +39,10 @@ static const char rcsid[] = "$Id: CascadeBG.c,v 1.128 1999/08/04 16:09:46 rwscot
 #include <Xm/MenuUtilP.h>
 #include <X11/Xfuncs.h>
 
-#ifndef __STDC__
-#include <varargs.h>
-#else
+#if STDC_HEADERS
 #include <stdarg.h>
+#else
+#include <varargs.h>
 #endif
 
 #include <XmI/DebugUtil.h>
@@ -255,7 +255,7 @@ static XmCacheClassPart cache_part = {
     _XmCascadeBCacheCompare
 };
 
-static int _XmArrowPixmapCacheCompare(XtPointer A, XtPointer B);
+int _XmArrowPixmapCacheCompare(XtPointer A, XtPointer B);
 
 static XmCacheClassPart arrow_pixmap_cache = {
     /* cache head part */
@@ -406,7 +406,7 @@ _XmCascadeBCacheCompare(XtPointer A, XtPointer B)
 		 sizeof(XmCascadeButtonGCacheObjPart));
 }
 
-static int
+int
 _XmArrowPixmapCacheCompare(XtPointer A, XtPointer B)
 {
     /* This will get confused by arrow direction and shadow thickness,
@@ -784,11 +784,42 @@ initialize(Widget request, Widget new_w,
 
 	if (CBG_Submenu(new_w))
 	{
-	    _LtLabelGAccomodateIndicator(new_w, request,
-		CBG_Cascade_width(new_w) + CASCADE_SPACING,
-		CBG_Cascade_height(new_w),
-		1, LabG_StringDirection(new_w) != XmSTRING_DIRECTION_L_TO_R,
-		False);
+	    int margin;
+
+	    /* Make sure there's enough room on the side
+	     * for the cascade pixmap.
+	     */
+	    margin = (CBG_Cascade_width(new_w) + CASCADE_SPACING)
+		- (LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+		   ? LabG_MarginRight(new_w) : LabG_MarginLeft(new_w));
+	    if (margin > 0)
+	    {
+		if (LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R)
+		    LabG_MarginRight(new_w) += margin;
+		else
+		{
+		    LabG_MarginLeft(new_w) += margin;
+		    LabG_TextRect_x(new_w) += margin;
+		}
+		if (!XtWidth(request))
+		    XtWidth(new_w) += margin;
+	    }
+
+	    /* Make sure there's enough room vertically */
+	    margin = CBG_Cascade_height(new_w) - (LabG_TextRect_height(new_w)
+						  + LabG_MarginTop(new_w)
+						  + LabG_MarginBottom(new_w));
+	    if (margin > 0)
+	    {
+		LabG_MarginTop(new_w) += margin >> 1;
+		LabG_MarginBottom(new_w) += (margin + 1) >> 1;
+		if (LabG_MenuType(new_w) != XmMENU_OPTION &&
+		    !XtHeight(request))
+		{
+		    LabG_TextRect_y(new_w) += margin >> 1;
+		    XtHeight(new_w) += margin;
+		}
+	    }
 	}
 
 	place_cascade(new_w);
@@ -816,7 +847,10 @@ resize(Widget w)
     (*superclass->rect_class.resize) (w);
 #undef superclass
 
-    place_cascade(w);
+    if (LabG_MenuType(w) != XmMENU_BAR)
+    {
+	place_cascade(w);
+    }
 }
 
 static void
@@ -1023,10 +1057,58 @@ set_values(Widget old, Widget request, Widget new_w,
 	 || LabG_MarginTop(new_w) != LabG_MarginTop(old)
 	 || LabG_MarginBottom(new_w) != LabG_MarginBottom(old)))
     {
-	_LtLabelGAccomodateIndicator(new_w, request,
-	    CBG_Cascade_width(new_w) + CASCADE_SPACING,
-	    CBG_Cascade_height(new_w),
-	    1, LabG_StringDirection(new_w) != XmSTRING_DIRECTION_L_TO_R, True);
+	int margin, tm, bm;
+
+	margin = CBG_Cascade_width(new_w) + CASCADE_SPACING
+	    - (LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+	       ? LabG_MarginRight(new_w) : LabG_MarginLeft(new_w));
+	if (margin && (margin > 0 ||
+	    (LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+	     ? LabG_MarginRight(new_w) == LabG_MarginRight(old)
+	     : LabG_MarginLeft(new_w) == LabG_MarginLeft(old))))
+	{
+	    if (LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R)
+		LabG_MarginRight(new_w) += margin;
+	    else
+	    {
+		LabG_MarginLeft(new_w) += margin;
+		LabG_TextRect_x(new_w) += margin;
+	    }
+	    if (LabG_RecomputeSize(new_w) || !XtWidth(request))
+		XtWidth(new_w) += margin;
+	}
+
+	margin = CBG_Cascade_height(new_w) - (LabG_TextRect_height(new_w)
+					      + LabG_MarginTop(new_w)
+					      + LabG_MarginBottom(new_w));
+	if (margin)
+	{
+	    tm = margin / 2;
+	    if (tm < (LabG_MarginTop(new_w) == LabG_MarginTop(old)
+		      ? XmDEFAULT_TOP_MARGIN - (int)LabG_MarginTop(new_w)
+		      : 0))
+		tm = (LabG_MarginTop(new_w) == LabG_MarginTop(old)
+		      ? XmDEFAULT_TOP_MARGIN - (int)LabG_MarginTop(new_w)
+		      : 0);
+	    LabG_MarginTop(new_w) += tm;
+	    if (LabG_MenuType(new_w) != XmMENU_OPTION &&
+		(LabG_RecomputeSize(new_w) || !XtHeight(request)))
+		XtHeight(new_w) += tm;
+
+	    bm = (margin + 1) / 2;
+	    if (bm < (LabG_MarginBottom(new_w) == LabG_MarginBottom(old)
+		      ? XmDEFAULT_BOTTOM_MARGIN - (int)LabG_MarginBottom(new_w)
+		      : 0))
+		bm = (LabG_MarginBottom(new_w) == LabG_MarginBottom(old)
+		      ? XmDEFAULT_BOTTOM_MARGIN - (int)LabG_MarginBottom(new_w)
+		      : 0);
+	    LabG_MarginBottom(new_w) += bm;
+	    if (LabG_RecomputeSize(new_w) || !XtHeight(request))
+		XtHeight(new_w) += bm;
+
+	    if (tm != bm)
+		LabG_TextRect_y(new_w) += (tm - bm) / 2;
+	}
 
 	refresh_needed = True;
     }
@@ -1720,7 +1802,7 @@ visual_change(Widget w,
     return False;
 }
 
-#ifdef __STDC__
+#if STDC_HEADERS
 static void
 MenuProcEntry(int proc, Widget w,...)
 {
