@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DrawnB.c,v 1.38 1999/07/10 11:52:49 rwscott Exp $
+ * $Id: DrawnB.c,v 1.39 1999/09/11 20:06:34 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DrawnB.c,v 1.38 1999/07/10 11:52:49 rwscott Exp $";
+static const char rcsid[] = "$Id: DrawnB.c,v 1.39 1999/09/11 20:06:34 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -365,8 +365,10 @@ set_values(Widget old, Widget request, Widget new_w,
        requested size.
        xdir top right hand logo button
        JHG: If there's something identifiable in the button, trust Label.
+       rws 11 Sep 1999
+       Don't trust Label. Take a look at drawnbutton/test9
      */
-    if (!Lab_TextRect_height(new_w) &&
+    if (/* !Lab_TextRect_height(new_w) && */
 	(XtWidth(new_w) != XtWidth(request) ||
 	 XtHeight(new_w) != XtHeight(request)))
     {
