@@ -1,7 +1,6 @@
-#define NEWGM
 /**
  *
- * $Id: Form.c,v 1.152 1999/07/20 01:17:11 rwscott Exp $
+ * $Id: Form.c,v 1.164 2000/01/04 23:44:13 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -23,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Form.c,v 1.152 1999/07/20 01:17:11 rwscott Exp $";
+static const char rcsid[] = "$Id: Form.c,v 1.164 2000/01/04 23:44:13 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -218,22 +217,22 @@ static XtResource formConstraintResources[] =
     },
     {
 	XmNtopPosition, XmCPosition, XmRInt,
-	sizeof(int), Offset(att[TOP].value),
+	sizeof(int), Offset(att[TOP].percent),
 	XmRImmediate, (XtPointer)0
     },
     {
 	XmNbottomPosition, XmCPosition, XmRInt,
-	sizeof(int), Offset(att[BOTTOM].value),
+	sizeof(int), Offset(att[BOTTOM].percent),
 	XmRImmediate, (XtPointer)0
     },
     {
 	XmNleftPosition, XmCPosition, XmRInt,
-	sizeof(int), Offset(att[LEFT].value),
+	sizeof(int), Offset(att[LEFT].percent),
 	XmRImmediate, (XtPointer)0
     },
     {
 	XmNrightPosition, XmCPosition, XmRInt,
-	sizeof(int), Offset(att[RIGHT].value),
+	sizeof(int), Offset(att[RIGHT].percent),
 	XmRImmediate, (XtPointer)0
     },
     {
@@ -615,12 +614,14 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     XtWidgetGeometry mr;
     int good = 0, ask = 0;
     Dimension wd, ht;
+    Dimension initial_width, initial_height;
     XtGeometryResult res;
 
 #define       Wants(x)        (request->request_mode & x)
 
     if ((request->request_mode & (CWX | CWY | CWWidth | CWHeight)) == 0)
     {
+	Form_ProcessingConstraints(f) = False;
 	*reply = *request;
 	return XtGeometryYes;
     }
@@ -640,16 +641,26 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     {
 	XmFormConstraints con = (XmFormConstraints)CoreConstraints(w);
 
-	if (mr.request_mode & CWWidth)
+	if (!Form_ProcessingConstraints(f))
 	{
-	    FCP_PrefW(con) = mr.width;
+	    if (mr.request_mode & CWWidth)
+	    {
+		FCP_PrefW(con) = mr.width;
+	    }
+	    if (mr.request_mode & CWHeight)
+	    {
+		FCP_PrefH(con) = mr.height;
+	    }
 	}
-	if (mr.request_mode & CWHeight)
+	else
 	{
-	    FCP_PrefH(con) = mr.height;
+	    Form_ProcessingConstraints(f) = False;
 	}
     }
 
+    initial_width = XtWidth(f);
+    initial_height = XtHeight(f);
+
     wd = XtIsRealized(f) ? 0 : XtWidth(f);
     ht = XtIsRealized(f) ? 0 : XtHeight(f);
 
@@ -662,8 +673,8 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 		       mr.width, mr.height, wd, ht));
     DEBUGOUT(XdbDebug2("GM", f, w,
 		       "GeometryManager: Form Layout makes child geo"
-		       " %dx%d form %dx%d %dx%d\n",
-		       mr.width, mr.height, wd, ht,
+		       " %dx%d%+i%+i form %dx%d %dx%d\n",
+		       mr.width, mr.height, mr.x, mr.y, wd, ht,
 		       XtWidth(f), XtHeight(f)));
 
     if ((res = _XmFormGeomRequest(f, &wd, &ht)) == XtGeometryYes)
@@ -715,15 +726,7 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     if (Wants(CWWidth))
     {
 	ask++;
-#ifdef NEWGM
-	/* rws 25 Nov 1998
-	   This should not be needed, however the layout is coming back
-	   "off by 1" sometimes. (main window budget)
-	 */
-	if ((mr.request_mode & CWWidth) && abs(mr.width - request->width) < 1)
-#else
 	if ((mr.request_mode & CWWidth) && mr.width == request->width)
-#endif
 	{
 	    good++;
 	}
@@ -758,6 +761,20 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     }
     else
     {
+	DEBUGOUT(XdbDebug2("GM", f, w,
+			   "GeometryManager: "
+			   " %dx%d %dx%d form %dx%d\n",
+			   wd, ht,
+			   initial_width, initial_height,
+			   XtWidth(f), XtHeight(f)
+			   ));
+	if (wd != initial_width ||
+	    ht != initial_height)
+	{
+	    XtWidth(f) = wd = initial_width;
+	    XtHeight(f) = ht = initial_height;
+	    _XmFormLayout(f, w, &mr, &wd, &ht);
+	}
 	_XmFormConfigureChildren(f, NULL, &mr);
     }
 
@@ -788,6 +805,26 @@ change_managed(Widget w)
 		      Form_InitialHeight(w) ? "InitialHeight" : "",
 		      XtWidth(w), XtHeight(w)));
 
+    {
+    int i;
+
+    	for (i = 0; i < MGR_NumChildren(w); i++)
+    	{
+	XmFormConstraints con = (XmFormConstraints)CoreConstraints(MGR_Children(w)[i]);
+
+	    if (XtIsManaged(MGR_Children(w)[i]))
+	    {
+    		if (FCP_PrefW(con) != XmINVALID_DIMENSION)
+    		{
+		    XtWidth(MGR_Children(w)[i]) = FCP_PrefW(con);
+    		}
+    		if (FCP_PrefH(con) != XmINVALID_DIMENSION)
+    		{
+		    XtHeight(MGR_Children(w)[i]) = FCP_PrefH(con);
+    		}
+	    }
+    	}
+    }
     wd = XtIsRealized(w) ? 0 : XtWidth(w);
     ht = XtIsRealized(w) ? 0 : XtHeight(w);
 
@@ -827,41 +864,35 @@ change_managed(Widget w)
     return;
 }
 
-static void
-_XmFixOffsets(Widget w)
+static int
+_XmGetOffset(Widget w, int i)
 {
-    XmFormConstraints con = (XmFormConstraints)CoreConstraints(w);
-    Widget f = XtParent(w);
-    int i;
+XmFormConstraints con = (XmFormConstraints)CoreConstraints(w);
+int offset;
 
-    for (i = 0; i < 4; i++)
+    if (FCP_Atta(con, i).offset == XmINVALID_DIMENSION)
     {
-	if (FCP_Atta(con, i).offset == XmINVALID_DIMENSION &&
-	    (i == TOP || i == BOTTOM))
+	if (FCP_Atta(con, i).type != XmATTACH_POSITION)
 	{
-	    if (FCP_Atta(con, i).type != XmATTACH_POSITION)
+	    if (i == TOP || i == BOTTOM)
 	    {
-		FCP_Atta(con, i).offset = Form_VertSpacing(f);
+		offset = Form_VertSpacing(XtParent(w));
 	    }
 	    else
 	    {
-		FCP_Atta(con, i).offset = 0;
+		offset = Form_HorizSpacing(XtParent(w));
 	    }
 	}
-
-	if (FCP_Atta(con, i).offset == XmINVALID_DIMENSION &&
-	    (i == LEFT || i == RIGHT))
+	else
 	{
-	    if (FCP_Atta(con, i).type != XmATTACH_POSITION)
-	    {
-		FCP_Atta(con, i).offset = Form_HorizSpacing(f);
-	    }
-	    else
-	    {
-		FCP_Atta(con, i).offset = 0;
-	    }
+	    offset = 0;
 	}
     }
+    else
+    {
+	offset = FCP_Atta(con, i).offset;
+    }
+    return(offset);
 }
 
 /*
@@ -879,14 +910,18 @@ constraint_initialize(Widget request, Widget new_w,
 		       "%s:constraint_initialize(%d) - %i args\n"
 		       "\trequest X %5i Y %5i W %5i H %5i\n"
 		       "\t  new_w X %5i Y %5i W %5i H %5i\n"
-		       "\t   Form W %5i H %5i\n",
+		       "\t   Form W %5i H %5i\n"
+		       "\t%s %s\n",
 		       __FILE__, __LINE__,
 		       *num_args,
 		       XtX(request), XtY(request),
 		       XtWidth(request), XtHeight(request),
 		       XtX(new_w), XtY(new_w),
 		       XtWidth(new_w), XtHeight(new_w),
-		       XtWidth(XtParent(new_w)), XtHeight(XtParent(new_w))));
+		       XtWidth(XtParent(new_w)), XtHeight(XtParent(new_w)),
+		       XtIsManaged(new_w) ? "Managed" : "Not Managed",
+		       XtIsRealized(new_w) ? "Realized" : "Not Realized"
+		       ));
     DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /* do this for all four contraints (TOP, BOTTOM, LEFT, RIGHT) */
@@ -905,9 +940,11 @@ constraint_initialize(Widget request, Widget new_w,
 	    break;
 
 	case XmATTACH_POSITION:
+	    /*
 	    FCP_Atta(con, i).percent =
 		(int)(((float)FCP_Atta(con, i).value /
 		       (float)Form_FractionBase(f)) * 100.0);
+	    */
 	    break;
 	}
     }
@@ -915,13 +952,10 @@ constraint_initialize(Widget request, Widget new_w,
     /* rws 2 Jul 1998
        Mozilla NavCenter window stop the left hand selector pane from coming
        up 12 pixels wide
+       form/test56
      */
-    /*
-       FCP_PrefW(con) = XtWidth(new_w);
-       FCP_PrefH(con) = XtHeight(new_w);
-     */
-    FCP_PrefW(con) = XtWidth(request);
-    FCP_PrefH(con) = XtHeight(request);
+    FCP_PrefW(con) = XmINVALID_DIMENSION;
+    FCP_PrefH(con) = XmINVALID_DIMENSION;
 }
 
 static Boolean
@@ -967,35 +1001,11 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 		       XtWidth(XtParent(new_w)), XtHeight(XtParent(new_w))));
     DEBUGOUT(XdbPrintArgList("DX", new_w, args, *num_args, False));
 
+    Form_ProcessingConstraints(f) = True;
     for (i = 0; i < 4; i++)
     {
-	if (FCP_Atta(ncon, i).value != FCP_Atta(old, i).value)
+	if (FCP_Atta(ncon, i).percent != FCP_Atta(old, i).percent)
 	{
-	    if (i == TOP || i == BOTTOM)
-	    {
-		if (XtHeight(f) != 0)
-		{
-		    FCP_Atta(ncon, i).percent =
-			(FCP_Atta(ncon, i).value / XtHeight(f));
-		}
-		else
-		{
-		    FCP_Atta(ncon, i).percent = 100;
-		}
-	    }
-	    else
-	    {
-		if (XtWidth(f) != 0)
-		{
-		    FCP_Atta(ncon, i).percent =
-			(FCP_Atta(ncon, i).value / XtWidth(f));
-		}
-		else
-		{
-		    FCP_Atta(ncon, i).percent = 100;
-		}
-	    }
-
 	    changed = True;
 	}
 
@@ -1005,7 +1015,28 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 	{
 	    changed = True;
 
+	    if (FCP_Atta(ncon, i).type != XmATTACH_WIDGET &&
+	        FCP_Atta(ncon, i).type != XmATTACH_OPPOSITE_WIDGET)
+	    {
+		FCP_Atta(ncon, i).w = NULL;
+	    }
 	    _XmCheckAttachedWidget(f, i, ncon);
+#if 0
+	    if (FCP_Atta(ncon, i).type == XmATTACH_NONE &&
+	        FCP_Atta(ncon, i).type != FCP_Atta(old, i).type &&
+	        (i == LEFT || i == RIGHT))
+	    {
+	    	fprintf(stderr, "%s %i %i\n", XtName(new_w), XtWidth(request), FCP_PrefW(ncon));
+/*	    	XtWidth(request) = FCP_PrefW(ncon); */
+	    }
+	    if (FCP_Atta(ncon, i).type == XmATTACH_NONE &&
+	        FCP_Atta(ncon, i).type != FCP_Atta(old, i).type &&
+	        (i == TOP || i == BOTTOM))
+	    {
+	    	fprintf(stderr, "%s %i %i\n", XtName(new_w), XtHeight(request), FCP_PrefH(ncon));
+/*	    	XtHeight(request) = FCP_PrefH(ncon); */
+	    }
+#endif
 	}
     }
 
@@ -1040,31 +1071,28 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 			   "constraint_set_values - refused resize\n"));
     }
 
-#if 1
     /* rws 24 Oct 1998 */
     /* scrolledwindow/test13 */
     /* form/test15 */
     /* Xinvest when centering in the frame */
-    if (changed && XtIsRealized(f) && XtIsManaged(f) && XtIsManaged(new_w))
+    DEBUGOUT(XdbDebug2("DX", (Widget)f, new_w,"%s %s %s %s\n",
+    	changed ? "True" : "False",
+    	XtIsRealized(f) ? "True" : "False",
+    	XtIsManaged(f) ? "True" : "False",
+    	XtIsManaged(new_w) ? "True" : "False"
+    	));
+    /* rws 19 Oct 1999
+       openDX splash screen (dx -edit) The pixmap should be centered
+     */
+    if (changed && XtIsRealized(f) && XtIsManaged(new_w))
     {
-#if 0
 	/* rws 23 Jun 1999
 	   OpenDX sub-classes XmForm to get WorkSpace, which is the main
 	   canvas for dx -edit. This has todo with the auto positioning that
 	   goes on in there. (Bug 107)
 	 */
-	XtWidgetGeometry request, reply;
-	CompositeClassRec *crec;
-
-	request.width = XtWidth(new_w);
-	request.height = XtHeight(new_w);
-	request.x = XtX(new_w);
-	request.y = XtY(new_w);
-	request.request_mode = CWX | CWY | CWWidth | CWHeight;
-	crec = (CompositeClassRec *)XtClass(f);
-	(crec->composite_class.geometry_manager)(new_w, &request, &reply);
-#else
 	Dimension wd, ht;
+	XtWidgetGeometry cg;
 
 	DEBUGOUT(XdbDebug(__FILE__, f,
 			  "GeomRequest in constraint_set_values\n"));
@@ -1077,26 +1105,37 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 	wd = XtIsRealized(f) ? 0 : XtWidth(f);
 	ht = XtIsRealized(f) ? 0 : XtHeight(f);
 
-	_XmFormLayout(f, NULL, NULL, &wd, &ht);
+	cg.request_mode = CWWidth | CWHeight;
+	cg.width = XtWidth(request);
+	cg.height = XtHeight(request);
+	_XmFormLayout(f, new_w, &cg, &wd, &ht);
 
 	if (_XmFormGeomRequest(f, &wd, &ht) == XtGeometryYes)
 	{
-	    _XmFormLayout(f, NULL, NULL, &wd, &ht);
-
-	    _XmFormConfigureChildren(f, NULL, NULL);
+	    _XmFormLayout(f, new_w, &cg, &wd, &ht);
+	    _XmFormConfigureChildren(f, new_w, &cg);
 	}
 	else
 	{
 	    wd = XtWidth(f);
 	    ht = XtHeight(f);
 
-	    _XmFormLayout(f, NULL, NULL, &wd, &ht);
-
-	    _XmFormConfigureChildren(f, NULL, NULL);
+	    _XmFormLayout(f, new_w, &cg, &wd, &ht);
+	    _XmFormConfigureChildren(f, new_w, &cg);
+	}
+	/*
+	if (XtWidth(current) == XtWidth(new_w) &&
+	    XtHeight(current) == XtHeight(new_w))
+	    */
+	{
+	    /* form/test56 */
+	    XtX(new_w)++;
 	}
-#endif
     }
-#endif
+    else
+    {
+	Form_ProcessingConstraints(f) = False;
+    }
 
     return False;
     return changed;
@@ -1409,30 +1448,56 @@ XmFormFindPreferred(Widget f, Widget cw, XtWidgetGeometry *cg)
 	       for mfm.  mfm is having problems in this area with or without
 	       the test... so I took it out for now.
 	     */
-	    if ( /*XtIsRealized(f) && */ FCP_PrefW(con) > 0)
+	    if (FCP_PrefW(con) != XmINVALID_DIMENSION &&
+	        cg && ((cg->request_mode & CWWidth) == CWWidth) &&
+	        !Form_ProcessingConstraints(f))
 	    {
 		SETW(FCP_PrefW(con) + 2 * XtBorderWidth(child));
 	    }
 	    else
 	    {
 		SETW(p.width + 2 * p.border_width);
-		FCP_PrefW(con) = p.width;
+		/* rws 15 Dec 1999
+		mainw/test16
+		*/
+	        if (FCP_PrefW(con) == XmINVALID_DIMENSION || (!Form_ProcessingConstraints(f) &&
+	            cg && ((cg->request_mode & CWWidth) == CWWidth)))
+	        {
+		    if (p.width > 0)
+		    {
+			FCP_PrefW(con) = p.width;
+		    }
+		}
 	    }
 
-	    if ( /*XtIsRealized(f) && */ FCP_PrefH(con) > 0)
+	    if (FCP_PrefH(con) != XmINVALID_DIMENSION &&
+	        cg && ((cg->request_mode & CWHeight) == CWHeight) &&
+	        !Form_ProcessingConstraints(f))
 	    {
 		SETH(FCP_PrefH(con) + 2 * XtBorderWidth(child));
 	    }
 	    else
 	    {
 		SETH(p.height + 2 * p.border_width);
-		FCP_PrefH(con) = p.height;
+		/* rws 15 Dec 1999
+		mainw/test16
+		*/
+	        if (FCP_PrefH(con) == XmINVALID_DIMENSION || (!Form_ProcessingConstraints(f) &&
+	            cg && ((cg->request_mode & CWHeight) == CWHeight)))
+	        {
+		    if (p.height > 0)
+		    {
+			FCP_PrefH(con) = p.height;
+		    }
+		}
 	    }
 	}
 	else
 	{
-	    FCP_PrefW(con) = 0;
-	    FCP_PrefH(con) = 0;
+	    /*
+	    FCP_PrefW(con) = XmINVALID_DIMENSION;
+	    FCP_PrefH(con) = XmINVALID_DIMENSION;
+	    */
 	}
     }
 
@@ -1464,7 +1529,7 @@ XmFormFindPreferred(Widget f, Widget cw, XtWidgetGeometry *cg)
     {									\
 	DEBUGOUT(XdbDebug0(__FILE__, (Widget)f, "%s(%d/%d,%d)\t",	\
 		 XdbAttachment2String(FCP_Atta(con, x).type),		\
-		 FCP_Atta(con, x).value, Form_FractionBase(f),		\
+		 FCP_Atta(con, x).percent, Form_FractionBase(f),		\
 		 FCP_Atta(con, x).offset));				\
     }									\
     else								\
@@ -1524,8 +1589,8 @@ XmFormPath(Widget f, Widget c, int dir, XmFormAttachment atta)
 	con = (XmFormConstraints)CoreConstraints(c);
 
 	atta[dir].type = FCP_Atta(con, dir).type;
-	atta[dir].value = FCP_Atta(con, dir).value;
-	atta[dir].offset = FCP_Atta(con, dir).offset;
+	atta[dir].percent = FCP_Atta(con, dir).percent;
+	atta[dir].offset = _XmGetOffset(c, dir);
 	atta[dir].w = c;
 	x = 0;
 	if (dir == TOP || dir == BOTTOM)
@@ -1545,16 +1610,16 @@ XmFormPath(Widget f, Widget c, int dir, XmFormAttachment atta)
 	    if (dir == TOP || dir == BOTTOM)
 	    {
 		x += BB_MarginHeight(f);
-		x += FCP_Atta(con, dir).offset;
+		x += _XmGetOffset(c, dir);
 		atta[dir].tempValue -= BB_MarginHeight(f);
-		atta[dir].tempValue -= FCP_Atta(con, dir).offset;
+		atta[dir].tempValue -= _XmGetOffset(c, dir);
 	    }
 	    else if (dir == LEFT || dir == RIGHT)
 	    {
 		x += BB_MarginWidth(f);
-		x += FCP_Atta(con, dir).offset;
+		x += _XmGetOffset(c, dir);
 		atta[dir].tempValue -= BB_MarginWidth(f);
-		atta[dir].tempValue -= FCP_Atta(con, dir).offset;
+		atta[dir].tempValue -= _XmGetOffset(c, dir);
 	    }
 	    break;
 	case XmATTACH_POSITION:
@@ -1566,11 +1631,11 @@ XmFormPath(Widget f, Widget c, int dir, XmFormAttachment atta)
 	    {
 	    	x += Form_HorizSpacing(f);
 	    }
-	    x += FCP_Atta(con, dir).offset;
+	    x += _XmGetOffset(c, dir);
 	    break;
 	case XmATTACH_WIDGET:
-	    x += FCP_Atta(con, dir).offset;
-	    atta[dir].tempValue -= FCP_Atta(con, dir).offset;
+	    x += _XmGetOffset(c, dir);
+	    atta[dir].tempValue -= _XmGetOffset(c, dir);
 	    break;
 	case XmATTACH_OPPOSITE_WIDGET:
 #if 1
@@ -1601,14 +1666,14 @@ XmFormPath(Widget f, Widget c, int dir, XmFormAttachment atta)
 		con1 = (XmFormConstraints)CoreConstraints(FCP_Atta(con, dir).w);
 
 		atta[dir].type = FCP_Atta(con1, dir).type;
-		atta[dir].value = FCP_Atta(con1, dir).value;
-		atta[dir].offset = FCP_Atta(con1, dir).offset;
+		atta[dir].percent = FCP_Atta(con1, dir).percent;
+		atta[dir].offset = _XmGetOffset(FCP_Atta(con, dir).w, dir);
 		atta[dir].w = FCP_Atta(con, dir).w;
-		x += FCP_Atta(con, dir).offset;
+		x += _XmGetOffset(c, dir);
 		atta[dir].tempValue = 0;
 		if (FCP_Atta(con1, dir).type != XmATTACH_NONE)
 		{
-		    x += FCP_Atta(con1, dir).offset;
+		    x += _XmGetOffset(FCP_Atta(con, dir).w, dir);
 		    if (FCP_Atta(con1, dir).type == XmATTACH_WIDGET)
 		    {
 			x += XmFormPath(f, FCP_Atta(con1, dir).w, dir, atta);
@@ -1658,8 +1723,8 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 		atta[BOTTOM].type == XmATTACH_POSITION)
 	    {
 		/* test 2 test30 nedit */
-		y = (y * Form_FractionBase(f) / (atta[BOTTOM].value -
-						 atta[TOP].value));
+		y = (y * Form_FractionBase(f) / (atta[BOTTOM].percent -
+						 atta[TOP].percent));
 		atta[TOP].tempValue = 0;
 	    }
 	    else if (atta[BOTTOM].type == XmATTACH_POSITION)
@@ -1668,17 +1733,17 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 
 		/* test18 test19 test27 */
 		y1 = (int)(((10 * (y) * Form_FractionBase(f) /
-			    atta[BOTTOM].value) + 5) / 10);
-		atta[TOP].tempValue = -y + (y1 * atta[BOTTOM].value) / Form_FractionBase(f);
+			    atta[BOTTOM].percent) + 5) / 10);
+		atta[TOP].tempValue = -y + (y1 * atta[BOTTOM].percent) / Form_FractionBase(f);
 		y = y1;
 	    }
 	    else if (atta[TOP].type == XmATTACH_POSITION)
 	    {
 		/* test28 test29 */
 		y = (int)(((10 * (y) * Form_FractionBase(f) /
-			  (Form_FractionBase(f) - atta[TOP].value)) + 5) / 10);
+			  (Form_FractionBase(f) - atta[TOP].percent)) + 5) / 10);
 		atta[TOP].tempValue = 0;
-		atta[TOP].tempValue = (y * atta[TOP].value) / Form_FractionBase(f);
+		atta[TOP].tempValue = (y * atta[TOP].percent) / Form_FractionBase(f);
 	    }
 	    else
 	    {
@@ -1716,8 +1781,8 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 	    if (atta[LEFT].type == XmATTACH_POSITION &&
 		atta[RIGHT].type == XmATTACH_POSITION)
 	    {
-		y = (y * Form_FractionBase(f) / (atta[RIGHT].value -
-						 atta[LEFT].value));
+		y = (y * Form_FractionBase(f) / (atta[RIGHT].percent -
+						 atta[LEFT].percent));
 		atta[LEFT].tempValue = 0;
 	    }
 	    else if (atta[RIGHT].type == XmATTACH_POSITION)
@@ -1725,18 +1790,18 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 	    int y1;
 
 		y1 = ((10 * (y) * Form_FractionBase(f) /
-		      atta[RIGHT].value) + 5) / 10;
+		      atta[RIGHT].percent) + 5) / 10;
 		atta[LEFT].tempValue = 0;
-		atta[LEFT].tempValue = -y + (y1 * atta[RIGHT].value) / Form_FractionBase(f);
+		atta[LEFT].tempValue = -y + (y1 * atta[RIGHT].percent) / Form_FractionBase(f);
 		y = y1;
 	    }
 	    else if (atta[LEFT].type == XmATTACH_POSITION)
 	    {
 		/* test13 test31 mgdiff */
 		y = ((10 * (y) * Form_FractionBase(f) /
-		      (Form_FractionBase(f) - atta[LEFT].value)) + 5) / 10;
+		      (Form_FractionBase(f) - atta[LEFT].percent)) + 5) / 10;
 		atta[LEFT].tempValue = 0;
-		atta[LEFT].tempValue = ((y * atta[LEFT].value) / Form_FractionBase(f));
+		atta[LEFT].tempValue = ((y * atta[LEFT].percent) / Form_FractionBase(f));
 	    }
 	    else
 	    {
@@ -1887,8 +1952,10 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	/*
 	 * INITIALIZE
 	 */
+	/*
 	FCP_WidthFromSide(con) = False;
 	FCP_HeightFromSide(con) = False;
+	*/
 
 #if 1
 	/* rws 3 Nov 1998
@@ -1921,13 +1988,15 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    if (Form_RubberPos(f) == True)
 	    {
 		FCP_Atta(con, LEFT).type = XmATTACH_POSITION;
-		FCP_Atta(con, LEFT).value = XtX(child);
+		FCP_Atta(con, LEFT).percent = XtX(child);
+		/*
 		FCP_Atta(con, LEFT).percent =
 		    (FCP_Atta(con, LEFT).value / Form_FractionBase(f));
+		    */
 
 		DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
 				   "Left set to ATTACH_POSITION %d\n",
-				   FCP_Atta(con, LEFT).value));
+				   FCP_Atta(con, LEFT).percent));
 	    }
 	    else
 	    {
@@ -1956,13 +2025,15 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    if (Form_RubberPos(f) == True)
 	    {
 		FCP_Atta(con, TOP).type = XmATTACH_POSITION;
-		FCP_Atta(con, TOP).value = XtY(child);
+		FCP_Atta(con, TOP).percent = XtY(child);
+		/*
 		FCP_Atta(con, TOP).percent =
 		    (FCP_Atta(con, TOP).value / Form_FractionBase(f));
+		    */
 
 		DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
 				   "Top set to ATTACH_POSITION %d\n",
-				   FCP_Atta(con, TOP).value));
+				   FCP_Atta(con, TOP).percent));
 	    }
 	    else
 	    {
@@ -1975,10 +2046,6 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    }
 	}
 #endif
-	/*
-	   If attachment offset are UNSPECIFIED use vertical/horizSpacing
-	 */
-	_XmFixOffsets(child);
     }
 
 #ifdef	PRINT_ATTACHMENT_REPORT
@@ -2092,18 +2159,18 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* TOP */
 	    case XmATTACH_FORM:
 		/* 1 3 4 5 6 8 9 10 11 12 13 14 15 17 19 20 21 22 23 24 25 27 */
-		if (FCP_Y(con) != FCP_Atta(con, TOP).offset +
+		if (FCP_Y(con) != _XmGetOffset(child, TOP) +
 		    BB_MarginHeight(f))
 		{
 		    if (!HeightChangeMode && FCP_Atta(con, BOTTOM).type != XmATTACH_NONE)
 		    {
 			/* 1 */
 			    SETH(FCP_Y(con) + FCP_H(con) -
-				 (FCP_Atta(con, TOP).offset +
+				 (_XmGetOffset(child, TOP) +
 				  BB_MarginHeight(f)));
 		    }
 
-		    SETY(FCP_Atta(con, TOP).offset + BB_MarginHeight(f));
+		    SETY(_XmGetOffset(child, TOP) + BB_MarginHeight(f));
 		}
 		break;
 
@@ -2111,17 +2178,17 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    case XmATTACH_OPPOSITE_FORM:
 		/* This is in Ted, an easy Rich Text processor */
 
-		if (FCP_Y(con) != fh + FCP_Atta(con, TOP).offset -
+		if (FCP_Y(con) != fh + _XmGetOffset(child, TOP) -
 		    BB_MarginHeight(f))
 		{
 		    if (!HeightChangeMode && FCP_Atta(con, BOTTOM).type != XmATTACH_NONE)
 		    {
 			    SETH(FCP_Y(con) + FCP_H(con) -
-				 (fh + FCP_Atta(con, TOP).offset -
+				 (fh + _XmGetOffset(child, TOP) -
 				  BB_MarginHeight(f)));
 		    }
 
-		    SETY(fh + FCP_Atta(con, TOP).offset - BB_MarginHeight(f));
+		    SETY(fh + _XmGetOffset(child, TOP) - BB_MarginHeight(f));
 		}
 		break;
 
@@ -2141,11 +2208,11 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 
 		if (FCP_Y(con) != (AttachedY(FCP_Atta(con, TOP).w) +
 				   AttachedHt(FCP_Atta(con, TOP).w) +
-				   FCP_Atta(con, TOP).offset))
+				   _XmGetOffset(child, TOP)))
 		{
 		    int yy = AttachedY(FCP_Atta(con, TOP).w) +
 		    AttachedHt(FCP_Atta(con, TOP).w) +
-		    FCP_Atta(con, TOP).offset;
+		    _XmGetOffset(child, TOP);
 
 		    if (!HeightChangeMode && FCP_Atta(con, BOTTOM).type != XmATTACH_NONE)
 		    {
@@ -2172,10 +2239,10 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		}
 
 		if (FCP_Y(con) != (AttachedY(FCP_Atta(con, TOP).w) +
-				   FCP_Atta(con, TOP).offset))
+				   _XmGetOffset(child, TOP)))
 		{
 		    int yy = AttachedY(FCP_Atta(con, TOP).w) +
-		    FCP_Atta(con, TOP).offset;
+		    _XmGetOffset(child, TOP);
 
 		    if (!HeightChangeMode && FCP_Atta(con, BOTTOM).type != XmATTACH_NONE)
 		    {
@@ -2190,9 +2257,9 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    case XmATTACH_POSITION:
 		/* 2 28 29 30 7 */
 		{
-		    int pos = (((10 * FCP_Atta(con, TOP).value * fh /
+		    int pos = (((10 * FCP_Atta(con, TOP).percent * fh /
 				 Form_FractionBase(f)) +
-				(10 * FCP_Atta(con, TOP).offset)) + 5) / 10;
+				(10 * _XmGetOffset(child, TOP))) + 5) / 10;
 
 		    if (FCP_Y(con) != pos)
 		    {
@@ -2229,8 +2296,8 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		 */
 		/* Xinvest */
 
-		FCP_Atta(con, TOP).value = (FCP_Y(con) +
-					    FCP_Atta(con, TOP).offset) *
+		FCP_Atta(con, TOP).percent = (FCP_Y(con) +
+					    _XmGetOffset(child, TOP)) *
 		    Form_FractionBase(f) / fh;
 		FCP_Atta(con, TOP).type = XmATTACH_POSITION;
 		FCP_Atta(con, TOP).offset = 0;
@@ -2253,18 +2320,18 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    case XmATTACH_FORM:
 		/* 1 14 15 16 17 20 21 23 25 28 29 3 31 5 6 8 */
 		if (FCP_Y(con) + FCP_H(con) != fh -
-		    FCP_Atta(con, BOTTOM).offset -
+		    _XmGetOffset(child, BOTTOM) -
 		    BB_MarginHeight(f))
 		{
 		    if (!HeightChangeMode && FCP_Atta(con, TOP).type != XmATTACH_NONE)
 		    {
 			/* 1 17 23 28 29 6 */
-			SETH(fh - FCP_Atta(con, BOTTOM).offset -
+			SETH(fh - _XmGetOffset(child, BOTTOM) -
 				 BB_MarginHeight(f) - FCP_Y(con));
 		    }
 		    if (!HeightChangeMode || FCP_Atta(con, TOP).type == XmATTACH_NONE)
 		    {
-			SETY(fh - FCP_Atta(con, BOTTOM).offset -
+			SETY(fh - _XmGetOffset(child, BOTTOM) -
 			     BB_MarginHeight(f) - FCP_H(con));
 		    }
 		}
@@ -2275,19 +2342,19 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* lxb */
 
 		if (FCP_H(con) + FCP_Y(con) != BB_MarginHeight(f) -
-		    FCP_Atta(con, BOTTOM).offset)
+		    _XmGetOffset(child, BOTTOM))
 		{
 		    if (!HeightChangeMode && FCP_Atta(con, TOP).type != XmATTACH_NONE)
 		    {
 			    SETH(BB_MarginHeight(f) -
-				 FCP_Atta(con, BOTTOM).offset -
+				 _XmGetOffset(child, BOTTOM) -
 				 FCP_Y(con));
 		    }
 		    if (!HeightChangeMode || FCP_Atta(con, TOP).type == XmATTACH_NONE)
 		    {
 
 			SETY(BB_MarginHeight(f) -
-			     FCP_Atta(con, BOTTOM).offset -
+			     _XmGetOffset(child, BOTTOM) -
 			     FCP_H(con));
 		    }
 		}
@@ -2310,18 +2377,18 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 
 		if (FCP_Y(con) + FCP_H(con) !=
 		    AttachedY(FCP_Atta(con, BOTTOM).w) -
-		    FCP_Atta(con, BOTTOM).offset)
+		    _XmGetOffset(child, BOTTOM))
 		{
 		    if (!HeightChangeMode && FCP_Atta(con, TOP).type != XmATTACH_NONE)
 		    {
 			/* 14 15 25 30 8 */
 			    SETH(AttachedY(FCP_Atta(con, BOTTOM).w) -
-				 FCP_Atta(con, BOTTOM).offset - FCP_Y(con));
+				 _XmGetOffset(child, BOTTOM) - FCP_Y(con));
 		    }
 		    if (!HeightChangeMode || FCP_Atta(con, TOP).type == XmATTACH_NONE)
 		    {
 		    SETY(AttachedY(FCP_Atta(con, BOTTOM).w) -
-			 FCP_Atta(con, BOTTOM).offset - FCP_H(con));
+			 _XmGetOffset(child, BOTTOM) - FCP_H(con));
 		    }
 		}
 		break;
@@ -2330,9 +2397,9 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    case XmATTACH_POSITION:
 		/* 18 19 2 27 30 7 */
 		{
-		    int pos = (((10 * FCP_Atta(con, BOTTOM).value * fh /
+		    int pos = (((10 * FCP_Atta(con, BOTTOM).percent * fh /
 				 Form_FractionBase(f)) -
-				(10 * FCP_Atta(con, BOTTOM).offset)) + 5) / 10;
+				(10 * _XmGetOffset(child, BOTTOM))) + 5) / 10;
 
 		    if (FCP_Y(con) + FCP_H(con) != pos)
 		    {
@@ -2378,7 +2445,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		if (FCP_Y(con) + FCP_H(con) !=
 		    (AttachedY(FCP_Atta(con, BOTTOM).w) +
 		     AttachedHt(FCP_Atta(con, BOTTOM).w) -
-		     FCP_Atta(con, BOTTOM).offset))
+		     _XmGetOffset(child, BOTTOM)))
 		{
 		    /* 12 13 21 8 */
 		    if (!HeightChangeMode && FCP_Atta(con, TOP).type != XmATTACH_NONE)
@@ -2386,13 +2453,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 			/* 12 13 8 */
 			    SETH(AttachedY(FCP_Atta(con, BOTTOM).w) +
 				 AttachedHt(FCP_Atta(con, BOTTOM).w) -
-				 FCP_Y(con) - FCP_Atta(con, BOTTOM).offset);
+				 FCP_Y(con) - _XmGetOffset(child, BOTTOM));
 		    }
 		    if (!HeightChangeMode || FCP_Atta(con, TOP).type == XmATTACH_NONE)
 		    {
 			SETY(AttachedY(FCP_Atta(con, BOTTOM).w) +
 			     AttachedHt(FCP_Atta(con, BOTTOM).w) -
-			     FCP_H(con) - FCP_Atta(con, BOTTOM).offset);
+			     FCP_H(con) - _XmGetOffset(child, BOTTOM));
 		    }
 		}
 		break;
@@ -2402,8 +2469,8 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		_XmWarning(child, "bottom ATTACH_SELF %s %d"
 			   " no test coverage", __FILE__, __LINE__);
 
-		FCP_Atta(con, BOTTOM).value = (FCP_Y(con) + FCP_H(con) +
-					       FCP_Atta(con, BOTTOM).offset) *
+		FCP_Atta(con, BOTTOM).percent = (FCP_Y(con) + FCP_H(con) +
+					       _XmGetOffset(child, BOTTOM)) *
 		    Form_FractionBase(f) / fh;
 
 		FCP_Atta(con, BOTTOM).type = XmATTACH_POSITION;
@@ -2426,18 +2493,18 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* LEFT */
 	    case XmATTACH_FORM:
 		/* 1 10 11 12 13 14 15 16 17 18 19 2 20 21 22 23 24 25 27 28 29 3 4 6 8 9 */
-		if (FCP_X(con) != FCP_Atta(con, LEFT).offset +
+		if (FCP_X(con) != _XmGetOffset(child, LEFT) +
 		    BB_MarginWidth(f))
 		{
 		    if (!WidthChangeMode && FCP_Atta(con, RIGHT).type != XmATTACH_NONE)
 		    {
 			/* 12 13 2 4 */
 			SETW(FCP_X(con) + FCP_W(con) -
-				 (FCP_Atta(con, LEFT).offset +
+				 (_XmGetOffset(child, LEFT) +
 				  BB_MarginWidth(f)));
 		    }
 
-		    SETX(FCP_Atta(con, LEFT).offset + BB_MarginWidth(f));
+		    SETX(_XmGetOffset(child, LEFT) + BB_MarginWidth(f));
 		}
 		break;
 
@@ -2445,9 +2512,9 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    case XmATTACH_POSITION:
 		/* 13 31 7 8 */
 		{
-		    int pos = (((10 * FCP_Atta(con, LEFT).value * fw /
+		    int pos = (((10 * FCP_Atta(con, LEFT).percent * fw /
 				 Form_FractionBase(f)) +
-				(10 * FCP_Atta(con, LEFT).offset)) + 5) / 10;
+				(10 * _XmGetOffset(child, LEFT))) + 5) / 10;
 
 		    if (FCP_X(con) != pos)
 		    {
@@ -2478,11 +2545,11 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		}
 		if (FCP_X(con) != (AttachedX(FCP_Atta(con, LEFT).w) +
 				   AttachedWd(FCP_Atta(con, LEFT).w) +
-				   FCP_Atta(con, LEFT).offset))
+				   _XmGetOffset(child, LEFT)))
 		{
 		    int xx = AttachedX(FCP_Atta(con, LEFT).w) +
 		    AttachedWd(FCP_Atta(con, LEFT).w) +
-		    FCP_Atta(con, LEFT).offset;
+		    _XmGetOffset(child, LEFT);
 
 		    if (!WidthChangeMode && FCP_Atta(con, RIGHT).type != XmATTACH_NONE)
 		    {
@@ -2509,10 +2576,10 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		    break;
 		}
 		if (FCP_X(con) != AttachedX(FCP_Atta(con, LEFT).w) +
-		    FCP_Atta(con, LEFT).offset)
+		    _XmGetOffset(child, LEFT))
 		{
 		    int xx = AttachedX(FCP_Atta(con, LEFT).w) +
-		    FCP_Atta(con, LEFT).offset;
+		    _XmGetOffset(child, LEFT);
 
 		    if (!WidthChangeMode && FCP_Atta(con, RIGHT).type != XmATTACH_NONE)
 		    {
@@ -2531,17 +2598,17 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* LEFT */
 	    case XmATTACH_OPPOSITE_FORM:
 		/* mfm */
-		if (FCP_X(con) != fw + FCP_Atta(con, LEFT).offset -
+		if (FCP_X(con) != fw + _XmGetOffset(child, LEFT) -
 		    BB_MarginWidth(f))
 		{
 		    if (!WidthChangeMode && FCP_Atta(con, RIGHT).type != XmATTACH_NONE)
 		    {
 			    SETW(FCP_X(con) + FCP_W(con) -
-				 (fw + FCP_Atta(con, LEFT).offset -
+				 (fw + _XmGetOffset(child, LEFT) -
 				  BB_MarginWidth(f)));
 		    }
 
-		    SETX(fw + FCP_Atta(con, LEFT).offset - BB_MarginWidth(f));
+		    SETX(fw + _XmGetOffset(child, LEFT) - BB_MarginWidth(f));
 		}
 		break;
 
@@ -2549,8 +2616,8 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    case XmATTACH_SELF:
 		/* Xinvest */
 
-		FCP_Atta(con, LEFT).value = (FCP_X(con) +
-					     FCP_Atta(con, LEFT).offset) *
+		FCP_Atta(con, LEFT).percent = (FCP_X(con) +
+					     _XmGetOffset(child, LEFT)) *
 		    Form_FractionBase(f) / fh;
 		FCP_Atta(con, LEFT).type = XmATTACH_POSITION;
 		FCP_Atta(con, LEFT).offset = 0;
@@ -2573,19 +2640,19 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    case XmATTACH_FORM:	/* GOOD EXAMPLE */
 		/* 1 10 11 12 13 14 15 2 22 25 3 4 5 6 8 9 */
 		if (FCP_X(con) + FCP_W(con) != fw -
-		    FCP_Atta(con, RIGHT).offset -
+		    _XmGetOffset(child, RIGHT) -
 		    BB_MarginWidth(f))
 		{
 		    if (!WidthChangeMode && FCP_Atta(con, LEFT).type != XmATTACH_NONE)
 		    {
 			/* 1 12 13 15 2 22 25 4 6 8 */
-			SETW(fw - FCP_Atta(con, RIGHT).offset -
+			SETW(fw - _XmGetOffset(child, RIGHT) -
 				 BB_MarginWidth(f) - FCP_X(con));
 		    }
 
 		    if (!WidthChangeMode || FCP_Atta(con, LEFT).type == XmATTACH_NONE)
 		    {
-			SETX(fw - FCP_Atta(con, RIGHT).offset -
+			SETX(fw - _XmGetOffset(child, RIGHT) -
 			     BB_MarginWidth(f) - FCP_W(con));
 		    }
 		}
@@ -2606,17 +2673,17 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		}
 		if (FCP_X(con) + FCP_W(con) !=
 		    AttachedX(FCP_Atta(con, RIGHT).w) -
-		    FCP_Atta(con, RIGHT).offset)
+		    _XmGetOffset(child, RIGHT))
 		{
 		    if (!WidthChangeMode && FCP_Atta(con, LEFT).type != XmATTACH_NONE)
 		    {
 			    SETW(AttachedX(FCP_Atta(con, RIGHT).w) -
-				 FCP_Atta(con, RIGHT).offset - FCP_X(con));
+				 _XmGetOffset(child, RIGHT) - FCP_X(con));
 		    }
 		    if (!WidthChangeMode || FCP_Atta(con, LEFT).type == XmATTACH_NONE)
 		    {
 			SETX(AttachedX(FCP_Atta(con, RIGHT).w) -
-			     FCP_Atta(con, RIGHT).offset - FCP_W(con));
+			     _XmGetOffset(child, RIGHT) - FCP_W(con));
 		    }
 		}
 		break;
@@ -2639,7 +2706,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		if (FCP_X(con) + FCP_W(con) !=
 		    (AttachedX(FCP_Atta(con, RIGHT).w) +
 		     AttachedWd(FCP_Atta(con, RIGHT).w) -
-		     FCP_Atta(con, RIGHT).offset))
+		     _XmGetOffset(child, RIGHT)))
 		{
 		    if (!WidthChangeMode && FCP_Atta(con, LEFT).type != XmATTACH_NONE)
 		    {
@@ -2647,13 +2714,13 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 			    SETW(AttachedX(FCP_Atta(con, RIGHT).w) +
 				 AttachedWd(FCP_Atta(con, RIGHT).w) -
 				 FCP_X(con) -
-				 FCP_Atta(con, RIGHT).offset);
+				 _XmGetOffset(child, RIGHT));
 		    }
 		    if (!WidthChangeMode || FCP_Atta(con, LEFT).type == XmATTACH_NONE)
 		    {
 			SETX(AttachedX(FCP_Atta(con, RIGHT).w) +
 			     AttachedWd(FCP_Atta(con, RIGHT).w) -
-			     FCP_W(con) - FCP_Atta(con, RIGHT).offset);
+			     FCP_W(con) - _XmGetOffset(child, RIGHT));
 		    }
 		}
 		break;
@@ -2662,9 +2729,9 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    case XmATTACH_POSITION:
 		/* 6 7 8 */
 		{
-		    int pos = (((10 * FCP_Atta(con, RIGHT).value * fw /
+		    int pos = (((10 * FCP_Atta(con, RIGHT).percent * fw /
 				 Form_FractionBase(f)) -
-				(10 * FCP_Atta(con, RIGHT).offset)) + 5) / 10;
+				(10 * _XmGetOffset(child, RIGHT))) + 5) / 10;
 
 		    if (FCP_X(con) + FCP_W(con) != pos)
 		    {
@@ -2696,16 +2763,16 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		/* mfm */
 
 		if (FCP_W(con) + FCP_X(con) != BB_MarginWidth(f) -
-		    FCP_Atta(con, RIGHT).offset)
+		    _XmGetOffset(child, RIGHT))
 		{
 		    if (!WidthChangeMode && FCP_Atta(con, LEFT).type != XmATTACH_NONE)
 		    {
-			    SETW(BB_MarginWidth(f) - FCP_Atta(con, RIGHT).offset -
+			    SETW(BB_MarginWidth(f) - _XmGetOffset(child, RIGHT) -
 				 FCP_X(con));
 		    }
 		    if (!WidthChangeMode || FCP_Atta(con, LEFT).type == XmATTACH_NONE)
 		    {
-		    SETX(BB_MarginWidth(f) - FCP_Atta(con, RIGHT).offset -
+		    SETX(BB_MarginWidth(f) - _XmGetOffset(child, RIGHT) -
 			 FCP_W(con));
 		    }
 		}
@@ -2716,8 +2783,8 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		_XmWarning(child, "right ATTACH_SELF %s %d"
 			   " no test coverage", __FILE__, __LINE__);
 
-		FCP_Atta(con, RIGHT).value = (FCP_X(con) + FCP_W(con) +
-					      FCP_Atta(con, RIGHT).offset) *
+		FCP_Atta(con, RIGHT).percent = (FCP_X(con) + FCP_W(con) +
+					      _XmGetOffset(child, RIGHT)) *
 		    Form_FractionBase(f) / fh;
 		FCP_Atta(con, RIGHT).type = XmATTACH_POSITION;
 		FCP_Atta(con, RIGHT).offset = 0;
