@@ -1,6 +1,6 @@
 /** 
  *
- * $Id: Frame.c,v 1.45 1998/11/08 01:49:17 rwscott Exp $
+ * $Id: Frame.c,v 1.47 2000/01/09 00:12:33 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Frame.c,v 1.45 1998/11/08 01:49:17 rwscott Exp $";
+static const char rcsid[] = "$Id: Frame.c,v 1.47 2000/01/09 00:12:33 rwscott Exp $";
 
 /*
  * Warning : this widget has a small amount of Motif 2.0 support.
@@ -465,7 +465,14 @@ resize(Widget w)
     }
 #endif
 
+    _XmClearShadowType(w, 
+    	Frame_OldWidth(w), 
+    	Frame_OldHeight(w),
+    	Frame_OldShadowThickness(w),
+    	0);
     _XmFrameConfigureChildren(w, NULL, NULL, XtWidth(w), XtHeight(w), True);
+    Frame_OldWidth(w) = XtWidth(w);
+    Frame_OldHeight(w) = XtHeight(w);
 }
 
 static void
