@@ -1,5 +1,5 @@
 /**
- * $Id: LTglob.c,v 1.8 1998/11/06 03:34:23 rwscott Exp $
+ * $Id: LTglob.c,v 1.9 2000/01/09 13:02:12 amai Exp $
  *
  * derived from GNU libc glob.c
  *
@@ -24,7 +24,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: LTglob.c,v 1.8 1998/11/06 03:34:23 rwscott Exp $";
+static const char rcsid[] = "$Id: LTglob.c,v 1.9 2000/01/09 13:02:12 amai Exp $";
 
 #if defined (_AIX) && !defined (__GNUC__)
 #pragma alloca
@@ -111,6 +111,13 @@ extern void qsort();
 
 #endif /* Standard headers.  */
 
+#ifdef __EMX__   /* -> OS/2 drive letters support */
+#define  INCL_DOS
+#define  INCL_DOSMISC
+#define  INCL_ERRORS
+#include <os2.h>
+#endif
+
 #ifndef	HAVE_STRCOLL
 #define	strcoll	strcmp
 #endif
@@ -702,3 +709,44 @@ glob_in_dir(const char *pattern, const char *directory, int flags,
     }
     return GLOB_NOSPACE;
 }
+
+#ifdef __EMX__
+/* Under OS/2 we may need to deal with drive letters ...*/
+
+static char *GetDriveMap(void)
+{
+  /* returns a string containing a list of all drives: e.g. "C: D: E:" */
+
+  static char drivelist[256];   /* so we'll never have a memory leak beyond 256 bytes */
+  static char tmpstr[4];
+  ULONG  CurDrive;
+  ULONG  DriveMap;
+  int    dnum;
+  int    start = 3;
+ 
+  drivelist[0] = '\0';
+  tmpstr[1] = ':';
+  tmpstr[2] = ' ';
+  tmpstr[3] = '\0';
+/*
+  DosError(0);
+*/
+  DosQueryCurrentDisk(&CurDrive, &DriveMap);
+  DriveMap>>=start-1;
+  for (dnum = start; dnum <= 26; dnum++) {
+     if ((DriveMap & (ULONG)1)) {
+        tmpstr[0] = (char)dnum+'A'-1;
+        strcat(drivelist, tmpstr);
+   }
+      DriveMap>>=1; 
+  }
+ 
+  if (drivelist[strlen(drivelist)-1]== ' ')
+    drivelist[strlen(drivelist)-1] = '\0';
+
+/*
+  DosError(1);
+*/
+  return drivelist;
+}
+#endif
