@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.105 1999/09/03 00:17:35 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.107 2000/01/03 22:32:57 danny Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.105 1999/09/03 00:17:35 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.107 2000/01/03 22:32:57 danny Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -1011,7 +1011,7 @@ set_values(Widget old, Widget request, Widget new_w,
 	}
 
 	/* Put the internal parts in the right places.
-	 * Since the size hasn't changed yes, use the old size.
+	 * Since the size hasn't changed yet, use the old size.
 	 * If it changes, resize will be called again by Xt.
 	 */
 
@@ -1602,80 +1602,6 @@ Help(Widget w, XEvent *event, String *params, Cardinal *num_params)
     _XmObjectUnlock(w);
 }
 
-void
-_LtLabelAccomodateIndicator(Widget w, Widget req, int iwidth, int iheight,
-			     int odd, int left, int sv)
-{
-    int tm, bm, minm;
-
-    /* Increase margins as necessary to fit an indicator for ToggleB and
-     * CascadeB.  They may be decreased as well if called from set_values.
-     * The external widget size may change if the requested size was default
-     * or recomputeSize is set in set_values.
-     */
-
-    /* Make room on the left or right side as needed */
-    if (left)
-    {
-	iwidth -= Lab_MarginLeft(w);
-	if (iwidth > 0 ||
-	    (iwidth && sv && Lab_MarginLeft(w) == Lab_MarginLeft(req)))
-	if (iwidth > 0)
-	{
-	    Lab_MarginLeft(w) += iwidth;
-	    Lab_TextRect_x(w) += iwidth;
-	    Lab_AccTextRect(w).x += iwidth;
-	    if (!XtWidth(req) || (sv && Lab_RecomputeSize(w)))
-		XtWidth(w) += iwidth;
-	}
-    }
-    else
-    {
-	iwidth -= Lab_MarginRight(w);
-	if (iwidth > 0 ||
-	    (iwidth && sv && Lab_MarginRight(w) == Lab_MarginRight(req)))
-	{
-	    Lab_MarginRight(w) += iwidth;
-	    if (!XtWidth(req) || (sv && Lab_RecomputeSize(w)))
-		XtWidth(w) += iwidth;
-	}
-    }
-
-    /* Make room vertically.  An odd-sized adjustment may put the odd bit
-     * on the bottom (or it may not).  The set_values case gets complex
-     * as the top may be modified but not the bottom, or vice versa.
-     */
-    iheight -= Lab_TextRect_height(w) + Lab_MarginTop(w) + Lab_MarginBottom(w);
-
-    tm = iheight / 2;
-    minm = sv && Lab_MarginTop(w) == Lab_MarginTop(req)
-	? XmDEFAULT_TOP_MARGIN - (int)Lab_MarginTop(w)
-	: 0;
-    if (tm < minm)
-	tm = minm;
-    Lab_MarginTop(w) += tm;
-
-    bm = (iheight + odd) / 2;
-    minm = sv && bm < Lab_MarginBottom(w) == Lab_MarginBottom(req)
-	? XmDEFAULT_BOTTOM_MARGIN - (int)Lab_MarginBottom(w)
-	: 0;
-    if (bm < minm)
-	bm = minm;
-    Lab_MarginBottom(w) += bm;
-
-    if (!XtHeight(req) || (sv && Lab_RecomputeSize(w)))
-    {
-	Lab_TextRect_y(w) += tm;
-	Lab_AccTextRect(w).y += tm;
-	XtHeight(w) += tm + bm;
-    }
-    else if (tm != bm)
-    {
-	Lab_TextRect_y(w) += (tm - bm) / 2;
-	Lab_AccTextRect(w).y += (tm - bm) / 2;
-    }
-}
-
 void
 _XmLabelGetPixmapSize(Widget w, Pixmap Pix,
 		      Dimension *width, Dimension *height)
@@ -1756,10 +1682,6 @@ Widget
 XmCreateLabel(Widget parent, char *name,
 	      Arg *arglist, Cardinal argcount)
 {
-	Widget	w;
-	_XmObjectLock(parent);
-	w = XtCreateWidget(name, xmLabelWidgetClass, parent,
+	return XtCreateWidget(name, xmLabelWidgetClass, parent,
 			  arglist, argcount);
-	_XmObjectUnlock(parent);
-	return w;
 }
