@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Manager.c,v 1.75 1999/08/15 17:17:09 rwscott Exp $
+ * $Id: Manager.c,v 1.80 2000/01/13 17:20:25 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Manager.c,v 1.75 1999/08/15 17:17:09 rwscott Exp $";
+static const char rcsid[] = "$Id: Manager.c,v 1.80 2000/01/13 17:20:25 rwscott Exp $";
 
 #include <LTconfig.h>
 
@@ -68,12 +68,13 @@ static const char rcsid[] = "$Id: Manager.c,v 1.75 1999/08/15 17:17:09 rwscott E
 
 static XmDirection get_direction(Widget w);
 
-static XmSpecifyLayoutDirectionTraitRec _XmManagerLayoutTraitRec = {
-    0, /* version */
-    get_direction /* getValue */
+static XmSpecifyLayoutDirectionTraitRec _XmManagerLayoutTraitRec =
+{
+    0,				/* version */
+    get_direction		/* getValue */
 };
 
-#endif 
+#endif
 
 /* Forward Declarations */
 
@@ -93,12 +94,12 @@ static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
 
 /*
-static XtGeometryResult geometry_manager(Widget w,
-					 XtWidgetGeometry *request,
-					 XtWidgetGeometry *reply);
+   static XtGeometryResult geometry_manager(Widget w,
+   XtWidgetGeometry *request,
+   XtWidgetGeometry *reply);
 
-static void changed_managed(Widget w);
-*/
+   static void changed_managed(Widget w);
+ */
 
 static void constraint_initialize(Widget request, Widget new_w,
 				  ArgList args, Cardinal *num_args);
@@ -337,16 +338,18 @@ static XtResource resources[] =
 #if XmVERSION > 1
 #if LESSTIF_BACK_COMPAT
 #else
-    ,{
+    ,
+    {
 	XmNpopupHandlerCallback, XmCCallback, XmRCallback,
 	sizeof(XtCallbackList), Offset(popup_handler_callback),
 	XtRPointer, (XtPointer)NULL
     }
 #endif
-    ,{
+    ,
+    {
 	XmNlayoutDirection, XmCLayoutDirection, XmRDirection,
 	sizeof(XmDirection), Offset(string_direction),
-	XmRImmediate, (XtPointer)((XmDirection)NULL) /* dynamic how? */
+	XmRImmediate, (XtPointer)((XmDirection) NULL)	/* dynamic how? */
     }
 #endif
 };
@@ -390,9 +393,9 @@ static XmSyntheticResource syn_resources[] =
     },
 #if XmVERSION > 1
     {
-	XmNstringDirection, 
+	XmNstringDirection,
 	sizeof(XmStringDirection), MOffset(string_direction),
-        _XmFromLayoutDirection, _XmToLayoutDirection
+	_XmFromLayoutDirection, _XmToLayoutDirection
     },
 #endif
 };
@@ -511,6 +514,7 @@ XmManagerClassRec xmManagerClassRec = {
 /* *INDENT-ON* */
 
 
+
 WidgetClass xmManagerWidgetClass = (WidgetClass)&xmManagerClassRec;
 
 static void
@@ -565,11 +569,11 @@ class_part_initialize(WidgetClass widget_class)
 	}
     }
     /*
-    else
-    {
-    	(*extptr)->accepts_objects = True;
-    }
-    */
+       else
+       {
+       (*extptr)->accepts_objects = True;
+       }
+     */
 
 
     if (widget_class != xmManagerWidgetClass)
@@ -582,7 +586,7 @@ class_part_initialize(WidgetClass widget_class)
 	if (mce && !*mce && sce && *sce)
 	{
 	    *mce =
-	      (XmManagerClassExtRec *)XtCalloc(1, sizeof(XmManagerClassExtRec));
+		(XmManagerClassExtRec *)XtCalloc(1, sizeof(XmManagerClassExtRec));
 	    (*mce)->next_extension = NULL;
 	    (*mce)->record_type = NULLQUARK;
 	    (*mce)->version = XmManagerClassExtVersion;
@@ -612,7 +616,7 @@ class_part_initialize(WidgetClass widget_class)
     _XmInitializeSyntheticResources(mwc->manager_class.syn_resources,
 				    mwc->manager_class.num_syn_resources);
     _XmInitializeSyntheticResources(mwc->manager_class.syn_constraint_resources,
-				    mwc->manager_class.num_syn_constraint_resources);
+			      mwc->manager_class.num_syn_constraint_resources);
 
     if (widget_class != xmManagerWidgetClass)
     {
@@ -630,15 +634,15 @@ class_part_initialize(WidgetClass widget_class)
 			  msc->manager_class.num_syn_constraint_resources);
     }
 
-    #if XmVERSION > 1
+#if XmVERSION > 1
     /* FIX ME: do I need to worry about subclasses */
     /* initialize traits */
-    if (! XmeTraitSet(xmManagerWidgetClass, XmQTspecifyLayoutDirection, 
-                &_XmManagerLayoutTraitRec))
+    if (!XmeTraitSet(xmManagerWidgetClass, XmQTspecifyLayoutDirection,
+		     &_XmManagerLayoutTraitRec))
     {
-        _XmWarning(NULL, "XmManager ClassInitialize: XmeTraitSet failed\n");
+	_XmWarning(NULL, "XmManager ClassInitialize: XmeTraitSet failed\n");
     }
-    #endif
+#endif
 }
 
 static void
@@ -754,7 +758,7 @@ initialize(Widget request,
 	   Cardinal *num_args)
 {
     XmManagerWidgetClass mwc = (XmManagerWidgetClass)new_w->core.widget_class;
-    /*XmManagerWidget	mw = (XmManagerWidget)new_w;*/
+    /*XmManagerWidget   mw = (XmManagerWidget)new_w; */
 
     MGR_EventHandlerAdded(new_w) = False;
     MGR_HighlightedWidget(new_w) = NULL;
@@ -801,7 +805,7 @@ initialize(Widget request,
     {
 	XtOverrideTranslations(new_w,
 			       (XtTranslations)mwc->
-					manager_class.translations);
+			       manager_class.translations);
     }
 
     /*
@@ -820,13 +824,13 @@ initialize(Widget request,
      */
     if (_XmGetBGPixmapName() != NULL)
     {
-        CoreBackgroundPixmap(new_w) =
-                XmGetPixmapByDepth(XtScreen(new_w),
-                                            _XmGetBGPixmapName(),
-                                            MGR_Foreground(new_w),
-                                            XtBackground(new_w),
-                                            CoreDepth(new_w));
-        _XmClearBGPixmap();
+	CoreBackgroundPixmap(new_w) =
+	    XmGetPixmapByDepth(XtScreen(new_w),
+			       _XmGetBGPixmapName(),
+			       MGR_Foreground(new_w),
+			       XtBackground(new_w),
+			       CoreDepth(new_w));
+	_XmClearBGPixmap();
     }
 
     CreateTopShadowGC(new_w);
@@ -841,9 +845,9 @@ initialize(Widget request,
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug("RWS", w,"%s:destroy(%d)\n",
-    	__FILE__, __LINE__
-    	));
+    DEBUGOUT(XdbDebug("RWS", w, "%s:destroy(%d)\n",
+		      __FILE__, __LINE__
+	     ));
     XtReleaseGC(w, MGR_TopShadowGC(w));
     XtReleaseGC(w, MGR_BottomShadowGC(w));
     XtReleaseGC(w, MGR_HighlightGC(w));
@@ -865,11 +869,11 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 {
     /* rws 13 Nov 1998
        This causes a parse error with gcc on AIX???
-    Boolean wz = False, hz = False; */
+       Boolean wz = False, hz = False; */
 
     DEBUGOUT(XdbDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
-    	__FILE__, __LINE__,
-    	XtWidth(w), XtHeight(w)));
+		      __FILE__, __LINE__,
+		      XtWidth(w), XtHeight(w)));
 
     /*
      * MLM 960524 - if you get an XError: widget has zero width or height.
@@ -885,9 +889,9 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 	XtHeight(w) = 1;
     }
 
-    *value_mask |= CWDontPropagate|CWBitGravity;
+    *value_mask |= CWDontPropagate | CWBitGravity;
     attributes->do_not_propagate_mask =
-	(KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask|
+	(KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask |
 	 PointerMotionMask);
     attributes->bit_gravity = NorthWestGravity;
 
@@ -896,15 +900,15 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 
     /* rws 7 Oct 1998
        drawingarea/test7 shows that this does not happen
-    if (wz)
-    {
-	XtWidth(w) = 0;
-    }
-    if (hz)
-    {
-	XtHeight(w) = 0;
-    }
-    */
+       if (wz)
+       {
+       XtWidth(w) = 0;
+       }
+       if (hz)
+       {
+       XtHeight(w) = 0;
+       }
+     */
 }
 
 static Boolean
@@ -919,16 +923,18 @@ set_values(Widget old,
     DEBUGOUT(XdbDebug(__FILE__, new_w, "SetValues()\n"));
 
 #if 0
-    if (XtWidth(new_w) != XtWidth(old) && XtWidth(new_w) == 0) {
+    if (XtWidth(new_w) != XtWidth(old) && XtWidth(new_w) == 0)
+    {
 	_XmWarning(new_w,
-		"XmManager SetValues: won't set width of widget to 0 (previous %d)\n",
-		XtWidth(old));
+	 "XmManager SetValues: won't set width of widget to 0 (previous %d)\n",
+		   XtWidth(old));
 	XtWidth(new_w) = XtWidth(old);
     }
-    if (XtHeight(new_w) != XtHeight(old) && XtHeight(new_w) == 0) {
+    if (XtHeight(new_w) != XtHeight(old) && XtHeight(new_w) == 0)
+    {
 	_XmWarning(new_w,
-		"XmManager SetValues: won't set height of widget to 0 (previous %d)\n",
-		XtHeight(old));
+	"XmManager SetValues: won't set height of widget to 0 (previous %d)\n",
+		   XtHeight(old));
 	XtHeight(new_w) = XtHeight(old);
     }
 #endif
@@ -937,18 +943,18 @@ set_values(Widget old,
     {
 	XSetWindowAttributes attr;
 
-        CoreBackgroundPixmap(new_w) =
-                XmGetPixmapByDepth(XtScreen(new_w),
-                                            _XmGetBGPixmapName(),
-                                            MGR_Foreground(new_w),
-                                            XtBackground(new_w),
-                                            CoreDepth(new_w));
+	CoreBackgroundPixmap(new_w) =
+	    XmGetPixmapByDepth(XtScreen(new_w),
+			       _XmGetBGPixmapName(),
+			       MGR_Foreground(new_w),
+			       XtBackground(new_w),
+			       CoreDepth(new_w));
 
 	attr.background_pixmap = CoreBackgroundPixmap(new_w);
 	XChangeWindowAttributes(XtDisplay(new_w), XtWindow(new_w),
-			        CWBackPixmap, &attr);
+				CWBackPixmap, &attr);
 
-        _XmClearBGPixmap();
+	_XmClearBGPixmap();
     }
 
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRUnitType),
@@ -1114,12 +1120,12 @@ _XmGadgetArm(Widget w,
 	_XmDispatchGadgetInput(g,
 			       event,
 			       XmARM_EVENT);
-	
+
 	MGR_SelectedGadget(w) = (XmGadget)g;
     }
     else
     {
-	if (_XmIsNavigable(w)) 
+	if (_XmIsNavigable(w))
 	{
 	    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 	}
@@ -1182,7 +1188,7 @@ _XmGadgetMultiArm(Widget w,
     }
     else
     {
-	if (_XmIsNavigable(w)) 
+	if (_XmIsNavigable(w))
 	{
 	    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 	}
@@ -1410,11 +1416,11 @@ _XmGadgetHelp(Widget w,
 
 void
 _XmManagerHelp(Widget w,
-	      XEvent *event,
-	      String *params,
-	      Cardinal *num_params)
+	       XEvent *event,
+	       String *params,
+	       Cardinal *num_params)
 {
-	_XmGadgetHelp(w, event, params, num_params);
+    _XmGadgetHelp(w, event, params, num_params);
 }
 
 void
@@ -1665,6 +1671,9 @@ _XmManagerEventHandler(Widget w,
 void
 _XmDestroyParentCallback(Widget w, XtPointer client, XtPointer call)
 {
+    DEBUGOUT(XdbDebug2("NEDIT", XtParent(w), w,
+		       "%s:_XmDestroyParentCallback(%d)\n",
+		       __FILE__, __LINE__));
     XtDestroyWidget(XtParent(w));
 }
 
@@ -1679,7 +1688,7 @@ _XmClearShadowType(Widget w,
 	if (XtWidth(w) >= old_width)
 	{
 	    XClearArea(XtDisplay(w), XtWindow(w),
-		       old_width - old_shadow_thickness - old_highlight_thickness,
+		    old_width - old_shadow_thickness - old_highlight_thickness,
 		       0, old_shadow_thickness,
 		       old_height - old_highlight_thickness, False);
 	}
@@ -1687,7 +1696,7 @@ _XmClearShadowType(Widget w,
 	{
 	    XClearArea(XtDisplay(w), XtWindow(w),
 		       0,
-		       old_height - old_shadow_thickness - old_highlight_thickness,
+		   old_height - old_shadow_thickness - old_highlight_thickness,
 		       old_width - old_highlight_thickness,
 		       old_shadow_thickness, False);
 	}
@@ -1735,6 +1744,11 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 		      "state %08x data %08x\n",
 		      event->type, event->xkey.keycode, event->xkey.state,
 		      client_data));
+    DEBUGOUT(XdbDebug("NMEM", w,
+		      "Got Accelerator key: type %d code %04x "
+		      "state %08x data %08x\n",
+		      event->type, event->xkey.keycode, event->xkey.state,
+		      client_data));
 
     if (!XmIsManager(w) ||
 	(event->type != KeyPress && event->type != KeyRelease))
@@ -1745,14 +1759,18 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
     }
 
     DEBUGOUT(XdbDebug(__FILE__, w,
-		      "Got Accelerator key: code: %08x state: %08x cd %08x %s\n",
+		    "Got Accelerator key: code: %08x state: %08x cd %08x %s\n",
+		      event->xkey.keycode, event->xkey.state, client_data,
+		      XtName(client_data)));
+    DEBUGOUT(XdbDebug("NMEM", w,
+		    "Got Accelerator key: code: %08x state: %08x cd %08x %s\n",
 		      event->xkey.keycode, event->xkey.state, client_data,
 		      XtName(client_data)));
 
     /* JAC added this test.  Occasionally in nedit the accelerators cause
        segfaults.  The comp value passed has 0 for all fields in the
        Widget.  e.g.:
-     
+
        (gdb) p *comp
 
        $10 = {core = {self = 0x0, widget_class = 0x0, parent = 0x0,
@@ -1770,15 +1788,16 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
        I suspect that "comp" may be the address of a widget that's been
        destroyed, and for some reason we're still getting the callback,
        but that's a WAG.
-    */
-   if (!comp->core.widget_class) {
-    _XmWarning(NULL, "XmManager _XmAcceleratorHandler: bad news!!!\n");
-      DEBUGOUT(XdbDebug(__FILE__, w,
-			"_XmAcceleratorHandler: called for w=%s"
-			" with invalid comp, ignoring\n",
-			XtName(w)));
-      return;
-      
+     */
+    if (!comp->core.widget_class)
+    {
+	_XmWarning(NULL, "XmManager _XmAcceleratorHandler: bad news!!!\n");
+	DEBUGOUT(XdbDebug(__FILE__, w,
+			  "_XmAcceleratorHandler: called for w=%s"
+			  " with invalid comp, ignoring\n",
+			  XtName(w)));
+	return;
+
     }
 
     /*
@@ -1790,20 +1809,20 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
      * no point in searching when we know that we can't do anything with
      * what we find.
      * --mu@trends.net, 1998.09.06 */
-    if(!XtIsSensitive(comp))
+    if (!XtIsSensitive(comp))
     {
-       DEBUGOUT(XdbDebug(__FILE__, w,
-                         "_XmAcceleratorHandler: %s is insensitive, ignoring\n",
-                         XtName(comp)));
-       return;
+	DEBUGOUT(XdbDebug(__FILE__, w,
+			"_XmAcceleratorHandler: %s is insensitive, ignoring\n",
+			  XtName(comp)));
+	return;
     }
 
     for (i = 0; i < MGR_NumKeyboardEntries(w); i++)
     {
-	if (/*MGR_KeyboardList(w)[i].eventType == event->type &&*/
-	    MGR_KeyboardList(w)[i].key == event->xkey.keycode &&
-	    MGR_KeyboardList(w)[i].modifiers == event->xkey.state &&
-	    MGR_KeyboardList(w)[i].component == client_data)
+	if (			/*MGR_KeyboardList(w)[i].eventType == event->type && */
+	       MGR_KeyboardList(w)[i].key == event->xkey.keycode &&
+	       MGR_KeyboardList(w)[i].modifiers == event->xkey.state &&
+	       MGR_KeyboardList(w)[i].component == client_data)
 	{
 	    Cardinal num_params = 0;
 
@@ -1812,42 +1831,70 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 			      "code %08x state %08x name: %s:%s mnem: %d\n",
 			      event->xkey.keycode, event->xkey.state,
 			      comp
-				? comp->core.widget_class->core_class.class_name
-				: "(nil)",
+			      ? comp->core.widget_class->core_class.class_name
+			      : "(nil)",
+			      comp ? XtName(comp) : "(nil)",
+			      MGR_KeyboardList(w)[i].isMnemonic));
+	    DEBUGOUT(XdbDebug("NMEM", w,
+			      "MATCH!!!! "
+			      "code %08x state %08x name: %s:%s mnem: %d\n",
+			      event->xkey.keycode, event->xkey.state,
+			      comp
+			      ? comp->core.widget_class->core_class.class_name
+			      : "(nil)",
 			      comp ? XtName(comp) : "(nil)",
 			      MGR_KeyboardList(w)[i].isMnemonic));
 
 	    /* Make sure XmGetPostedFromWidget works: ? */
 
-	    if (XmIsPrimitive(MGR_KeyboardList(w)[i].component))
+	    if (!MGR_KeyboardList(w)[i].isMnemonic || RC_Type(XtParent(comp)) == XmMENU_BAR ||
+		(MGR_KeyboardList(w)[i].isMnemonic && XtParent(comp) == RC_PopupPosted(w))
+		)
 	    {
-		act = PrimC_ArmAndActivate(XtClass(comp));
+		DEBUGOUT(XdbDebug2(__FILE__, w, MGR_KeyboardList(w)[i].component,
+				   "menu showing\n"));
+		DEBUGOUT(XdbDebug2("NMEM", w, MGR_KeyboardList(w)[i].component,
+				   "menu showing\n"));
+		if (XmIsPrimitive(MGR_KeyboardList(w)[i].component))
+		{
+		    act = PrimC_ArmAndActivate(XtClass(comp));
 
-		(*act)(comp, event, NULL, &num_params);
-	    }
-	    else if (XmIsGadget(MGR_KeyboardList(w)[i].component))
-	    {
-		act = GC_ArmAndActivate(XtClass(comp));
+		    (*act) (comp, event, NULL, &num_params);
+		}
+		else if (XmIsGadget(MGR_KeyboardList(w)[i].component))
+		{
+		    act = GC_ArmAndActivate(XtClass(comp));
 
-		(*act)(comp, event, NULL, &num_params);
-	    }
-	    else if (XmIsRowColumn(w))
-	    {
-		act = RCClass_ArmAndActivate(XtClass(comp));
+		    (*act) (comp, event, NULL, &num_params);
+		}
+		else if (XmIsRowColumn(w))
+		{
+		    act = RCClass_ArmAndActivate(XtClass(comp));
 
-		(*act)(comp, event, NULL, &num_params);
+		    (*act) (comp, event, NULL, &num_params);
+		}
+		else
+		{
+		    DEBUGOUT(XdbDebug(__FILE__, MGR_KeyboardList(w)[i].component,
+				      "Not a widget/gadget\n"));
+
+		    XtCallActionProc(comp, "Activate", event, NULL, 0);
+		}
+		*cont = False;
+		break;
 	    }
 	    else
 	    {
-		DEBUGOUT(XdbDebug(__FILE__, MGR_KeyboardList(w)[i].component,
-				  "Not a widget/gadget\n"));
-
-		XtCallActionProc(comp, "Activate", event, NULL, 0);
+		DEBUGOUT(XdbDebug2(__FILE__, w, MGR_KeyboardList(w)[i].component,
+				   "menu is not showing\n"));
+		DEBUGOUT(XdbDebug2("NMEM", w, MGR_KeyboardList(w)[i].component,
+				   "menu is not showing\n"));
 	    }
 
+	    /*
 	    *cont = False;
-
 	    break;
+	    */
 	}
     }
 }
@@ -1900,10 +1947,17 @@ AddKeyboardEntry(Widget m, Widget w, XmKeyboardData *item)
     MGR_NumKeyboardEntries(m)++;
 
     DEBUGOUT(XdbDebug(__FILE__, m,
-		      "GrabKey: key %04x Sym: %04x modifiers %08x\n",
-		      item->key, item->keysym, item->modifiers));
+		      "GrabKey: key %04x Sym: %04x modifiers %08x %s %s\n",
+		      item->key, item->keysym, item->modifiers,
+		      item->needGrab ? "Grabbing" : "Not Grabbing",
+		      item->isMnemonic ? "mnemonic" : "not mnemonic"));
+    DEBUGOUT(XdbDebug("NMEM", m,
+		      "GrabKey: key %04x Sym: %04x modifiers %08x %s %s\n",
+		      item->key, item->keysym, item->modifiers,
+		      item->needGrab ? "Grabbing" : "Not Grabbing",
+		      item->isMnemonic ? "mnemonic" : "not mnemonic"));
 
-    XtInsertEventHandler(m, item->eventType /*KeyPressMask|KeyReleaseMask*/,
+    XtInsertEventHandler(m, item->eventType /*KeyPressMask|KeyReleaseMask */ ,
 			 False, _XmAcceleratorHandler,
 			 (XtPointer)w, XtListHead);
 
@@ -1946,7 +2000,7 @@ DeleteKeyboardEntry(Widget m, Widget w, Boolean isMnemonic)
 		bcopy(&MGR_KeyboardList(m)[i + 1],
 		      &MGR_KeyboardList(m)[i],
 		      sizeof(XmKeyboardData) *
-			(MGR_NumKeyboardEntries(m) - i - 1));
+		        (MGR_NumKeyboardEntries(m) - i - 1));
 
 		bzero(&MGR_KeyboardList(m)[MGR_NumKeyboardEntries(m) - 1],
 		      sizeof(XmKeyboardData));
@@ -1974,6 +2028,8 @@ _XmManagerInstallAccelerator(Widget m, Widget w, String s)
 
     DEBUGOUT(XdbDebug2(__FILE__, m, w,
 		       "_XmManagerInstallAccelerator(%s)\n", s));
+    DEBUGOUT(XdbDebug2("NMEM", m, w,
+		       "_XmManagerInstallAccelerator(%s)\n", s));
 
     /* Do we really have an accelerator ? */
     if (s == NULL || strlen(s) == 0)
@@ -1998,12 +2054,19 @@ _XmManagerInstallAccelerator(Widget m, Widget w, String s)
 
 	if (XtIsSubclass(sh, applicationShellWidgetClass) ||
 	    XtIsSubclass(sh, xmDialogShellWidgetClass) ||
+	    XtIsSubclass(sh, transientShellWidgetClass) ||
 	    XtIsSubclass(sh, topLevelShellWidgetClass))
 	{
 	    break;
 	}
     }
 
+    if (!XmIsManager(mw))
+    {
+	return;
+    }
+
+
     DEBUGOUT(XdbDebug(__FILE__, w,
 		      "_XmManagerInstallAccelerator found manager %s\n",
 		      XtName(mw)));
@@ -2019,8 +2082,31 @@ _XmManagerInstallAccelerator(Widget m, Widget w, String s)
     data.key = XKeysymToKeycode(XtDisplay(w), data.keysym);
     data.modifiers = modifiers;
     data.needGrab = True;
+    if (data.key == 0)
+    {
+#if 0
+    KeySym real_sym;
+    Modifiers real_mod;
 
-    AddKeyboardEntry(mw, w, &data);
+	_XmVirtualToActualKeysym(XtDisplay(w), data.keysym, &real_sym, &real_mod);
+	if (real_sym == NoSymbol && real_mod == (Modifiers)0)
+	{
+	    _XmWarning(m, "%s:_XmManagerInstallAccelerator(%d) - Could not convert >%s< to a KeyCode\n    %s -> %08x -> %i", 
+		__FILE__, __LINE__, s,
+		s, data.keysym, data.key);
+	}
+	else
+	{
+	    data.key = XKeysymToKeycode(XtDisplay(w), real_sym);
+	    data.modifiers = real_mod;
+	    AddKeyboardEntry(mw, w, &data);
+	}
+#endif
+    }
+    else
+    {
+	AddKeyboardEntry(mw, w, &data);
+    }
 }
 
 void
@@ -2034,6 +2120,9 @@ _XmManagerUninstallAccelerator(Widget m, Widget w)
 	sh = XtParent(mw);
 	if (XtIsSubclass(sh, applicationShellWidgetClass) ||
 	    XtIsSubclass(sh, xmDialogShellWidgetClass) ||
+	    XtIsSubclass(sh, transientShellWidgetClass) ||
+		XtIsSubclass(sh, transientShellWidgetClass) ||
+		XtIsSubclass(sh, transientShellWidgetClass) ||
 	    XtIsSubclass(sh, topLevelShellWidgetClass))
 	{
 	    break;
@@ -2061,31 +2150,40 @@ _XmManagerInstallMnemonic(Widget m, Widget w, KeySym mn)
     }
 
     DEBUGOUT(XdbDebug2(__FILE__, m, w, "_XmManagerInstallMnemonic(%c)\n", mn));
+    DEBUGOUT(XdbDebug2("NMEM", m, w, "_XmManagerInstallMnemonic(%c)\n", mn));
 
     /* If we're dealing with the menu bar, store the stuff in the highest
      * manager widget that you can find (with all the accelerators) */
     if (XmIsRowColumn(m) &&
-	(RC_Type(m) == XmMENU_BAR || RC_Type(m) == XmMENU_OPTION))
+	(RC_Type(m) == XmMENU_PULLDOWN || RC_Type(m) == XmMENU_BAR || RC_Type(m) == XmMENU_OPTION))
     {
 	Widget mw, sh;
 	XmModifierMaskSetReference mods =
-		_XmGetModifierMappingsForDisplay(XtDisplay(w));
+	_XmGetModifierMappingsForDisplay(XtDisplay(w));
 
-	need_grab = True;
-	modifiers |= mods[ALTModifier];
+	if (RC_Type(m) == XmMENU_PULLDOWN)
+	{
+	}
+	else
+	{
+	    need_grab = True;
+	    modifiers |= mods[ALTModifier];
+	}
 
 	for (mw = m; XtParent(mw); mw = sh)
 	{
 	    sh = XtParent(mw);
 	    if (XtIsSubclass(sh, applicationShellWidgetClass) ||
 		XtIsSubclass(sh, xmDialogShellWidgetClass) ||
-		XtIsSubclass(sh, topLevelShellWidgetClass))
+		XtIsSubclass(sh, topLevelShellWidgetClass) ||
+		(RC_Type(m) != XmMENU_BAR && XmIsRowColumn(mw) && RC_Type(mw) == XmMENU_BAR))
 	    {
 		break;
 	    }
 	}
 
 	DEBUGOUT(XdbDebug2(__FILE__, mw, w, "Store mnemonic info\n"));
+	DEBUGOUT(XdbDebug2("NMEM", mw, w, "Store mnemonic info\n"));
 	m = mw;
     }
 
@@ -2093,7 +2191,7 @@ _XmManagerInstallMnemonic(Widget m, Widget w, KeySym mn)
     {
 	/* Put the data in the entry */
 	data.component = w;
-	data.eventType = KeyPressMask;
+	data.eventType = KeyPress;
 	data.keysym = mn;
 	data.isMnemonic = True;
 	data.key = XKeysymToKeycode(XtDisplay(w), data.keysym);
@@ -2164,44 +2262,44 @@ widget_navigable(Widget w)
 void
 _XmManagerHighlightPixmapDefault(Widget w, int offset, XrmValue *value)
 {
-	static Pixmap pixmap;  
+    static Pixmap pixmap;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmManagerHighlightPixmapDefault\n"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmManagerHighlightPixmapDefault\n"));
 
-	pixmap = XmUNSPECIFIED_PIXMAP;
+    pixmap = XmUNSPECIFIED_PIXMAP;
 
-	value->addr = (char *) &pixmap;
-	value->size = sizeof (Pixmap);
+    value->addr = (char *)&pixmap;
+    value->size = sizeof(Pixmap);
 
-	if (MGR_HighlightColor(w) == XtBackground(w))
-		pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
-			MGR_HighlightColor(w),
-			MGR_Foreground(w),
-			CoreDepth(w));
+    if (MGR_HighlightColor(w) == XtBackground(w))
+	pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
+				    MGR_HighlightColor(w),
+				    MGR_Foreground(w),
+				    CoreDepth(w));
 }
 
 void
 _XmManagerTopShadowPixmapDefault(Widget w, int offset, XrmValue *value)
 {
-	static Pixmap pixmap;
+    static Pixmap pixmap;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmManagerTopShadowPixmapDefault\n"));
+    DEBUGOUT(XdbDebug(__FILE__, w, "_XmManagerTopShadowPixmapDefault\n"));
 
-	pixmap = XmUNSPECIFIED_PIXMAP;
+    pixmap = XmUNSPECIFIED_PIXMAP;
 
-	value->addr = (char *) &pixmap;
-	value->size = sizeof (Pixmap);
+    value->addr = (char *)&pixmap;
+    value->size = sizeof(Pixmap);
 
-	if (MGR_TopShadowColor(w) == XtBackground(w))
-		pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
-			MGR_TopShadowColor(w),
-			MGR_Foreground(w),
-			CoreDepth(w));
-	else if (DefaultDepthOfScreen(XtScreen(w)) == 1)
-		pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
-			MGR_TopShadowColor(w),
-			XtBackground(w),
-			CoreDepth(w));
+    if (MGR_TopShadowColor(w) == XtBackground(w))
+	pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
+				    MGR_TopShadowColor(w),
+				    MGR_Foreground(w),
+				    CoreDepth(w));
+    else if (DefaultDepthOfScreen(XtScreen(w)) == 1)
+	pixmap = XmGetPixmapByDepth(XtScreen(w), "50_foreground",
+				    MGR_TopShadowColor(w),
+				    XtBackground(w),
+				    CoreDepth(w));
 }
 
 static void
@@ -2212,7 +2310,7 @@ _XmManagerExportX(Widget widget, int offset, XtArgVal *value)
 
     if (XtIsManaged(widget) && XtParent(widget) && XmIsVendorShell(XtParent(widget)))
     {
-    	*value = XtX(XtParent(widget));
+	*value = XtX(XtParent(widget));
     }
     converted_value = XmConvertUnits(widget,
 				     XmHORIZONTAL,
@@ -2231,7 +2329,7 @@ _XmManagerExportY(Widget widget, int offset, XtArgVal *value)
 
     if (XtIsManaged(widget) && XtParent(widget) && XmIsVendorShell(XtParent(widget)))
     {
-    	*value = XtY(XtParent(widget));
+	*value = XtY(XtParent(widget));
     }
     converted_value = XmConvertUnits(widget,
 				     XmVERTICAL,
@@ -2243,16 +2341,16 @@ _XmManagerExportY(Widget widget, int offset, XtArgVal *value)
 }
 
 #if XmVERSION > 1
-static XmDirection get_direction(Widget w)
+static XmDirection 
+get_direction(Widget w)
 {
     /* 
-      This macro obtains the layout direction resource even
-      though it looks like the string direction resource. 
-      In 2.0 layout direction replaced string direction
-      but the identifier in the internal data structure stayed
-      the same.  Everyone still with me. 8)
-    */
+       This macro obtains the layout direction resource even
+       though it looks like the string direction resource. 
+       In 2.0 layout direction replaced string direction
+       but the identifier in the internal data structure stayed
+       the same.  Everyone still with me. 8)
+     */
     return MGR_StringDirection(w);
 }
 #endif
-
