@@ -1,6 +1,6 @@
 /**
  *
- * $Id: PushBG.c,v 1.75 1999/08/14 09:59:26 danny Exp $
+ * $Id: PushBG.c,v 1.76 1999/11/11 13:43:22 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PushBG.c,v 1.75 1999/08/14 09:59:26 danny Exp $";
+static const char rcsid[] = "$Id: PushBG.c,v 1.76 1999/11/11 13:43:22 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -41,10 +41,10 @@ static const char rcsid[] = "$Id: PushBG.c,v 1.75 1999/08/14 09:59:26 danny Exp
 #include <X11/Xfuncs.h>
 #include <X11/ShellP.h>
 
-#ifndef __STDC__
-#include <varargs.h>
-#else
+#if STDC_HEADERS
 #include <stdarg.h>
+#else
+#include <varargs.h>
 #endif
 
 #include <XmI/DebugUtil.h>
@@ -1834,7 +1834,7 @@ _XmClearBGCompatibility(Widget w)
     _XmObjectUnlock(w);
 }
 
-#ifdef __STDC__
+#if STDC_HEADERS
 static void
 MenuProcEntry(int proc, Widget w,...)
 {
