@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ResConvert.c,v 1.50 1999/08/27 09:13:25 danny Exp $
+ * $Id: ResConvert.c,v 1.54 1999/11/11 13:43:23 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,13 +22,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ResConvert.c,v 1.50 1999/08/27 09:13:25 danny Exp $";
+static const char rcsid[] = "$Id: ResConvert.c,v 1.54 1999/11/11 13:43:23 rwscott Exp $";
 
 #include <LTconfig.h>
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef __STDC__
+#if STDC_HEADERS
 #include <stdarg.h>
 #else
 #include <varargs.h>
@@ -107,6 +107,15 @@ static void destroy_xm_string_table(XtAppContext app, XrmValue *to,
 				    Cardinal *num_args);
 static void get_unit_type(Widget w, Cardinal *size, XrmValue *val);
 
+#if XmVERSION > 1
+static Boolean _XmCvtStringToTabList(Display *display, XrmValue *args,
+		       Cardinal *num_args, XrmValue *from, XrmValue *to,
+		       XtPointer *converter_data);
+static Boolean _XmCvtStringToRenderTable(Display *display, XrmValue *args,
+		       Cardinal *num_args, XrmValue *from, XrmValue *to,
+		       XtPointer *converter_data);
+#endif
+
 int	xmUseVersion = 0;
 
 /* please be careful if you're editting these arrays, as
@@ -528,6 +537,7 @@ static unsigned char spb_arrow_layout_values[] =
     XmARROWS_SPLIT
 };
 
+#if XmVERSION > 1
 /* Notebook */
 static char *nb_child_types[] =
 {
@@ -548,6 +558,7 @@ static unsigned char nb_child_type_values[] =
     XmSTATUS_AREA, 
     XmPAGE_SCROLLER
 };
+#endif
 
 static char *indicator_on[] =
 {
@@ -957,6 +968,23 @@ _XmRegisterConverters(void)
 		       0,	/* number of args to converter routine */
 		       XtCacheNone,	/* caching instructions */
 		       NULL);	/* destructor function */
+#if XmVERSION > 1
+    XtSetTypeConverter(XtRString,	/* source type */
+		       XmRTabList,	/* target type */
+		       _XmCvtStringToTabList,	/* converter routine */
+		       NULL,	/* args for converter routine */
+		       0,	/* number of args to converter routine */
+		       XtCacheNone,	/* caching instructions */
+		       NULL);	/* destructor function */
+
+    XtSetTypeConverter(XtRString,	/* source type */
+		       XmRRenderTable,	/* target type */
+		       _XmCvtStringToRenderTable,	/* converter routine */
+		       NULL,	/* args for converter routine */
+		       0,	/* number of args to converter routine */
+		       XtCacheNone,	/* caching instructions */
+		       NULL);	/* destructor function */
+#endif
 
 #ifdef NONSTANDARD_CONVERTERS
     _XmRegisterNSEConverters();
@@ -967,7 +995,7 @@ _XmRegisterConverters(void)
  * error convenience
  */
 void
-#ifdef __STDC__
+#if STDC_HEADERS
 _XmError(Widget w, char *fmt,...)
 {
     va_list arg_list;
@@ -1009,7 +1037,7 @@ _XmError(w, fmt, va_alist)
  * bugs of cource :-(
  */
 void
-#ifdef __STDC__
+#if STDC_HEADERS
 _XmWarning(Widget w, char *fmt,...)
 {
     va_list arg_list;
@@ -2534,7 +2562,7 @@ get_unit_type(Widget w, Cardinal *size, XrmValue *val)
 {
     static unsigned char ut;
 
-    val->addr = &ut;
+    val->addr = (XPointer)&ut;
     val->size = sizeof ut;
     ut = _XmGetUnitType(w);
 }
@@ -2573,5 +2601,28 @@ XmConvertStringToUnits(Screen *screen, String spec, int orientation,
 	default:
 		return 0;
 	}
+    return 0;
+}
+
+static Boolean
+_XmCvtStringToTabList(Display *display,
+		       XrmValue *args,
+		       Cardinal *num_args,
+		       XrmValue *from,
+		       XrmValue *to,
+		       XtPointer *converter_data)
+{
+	return False;
+}
+
+static Boolean
+_XmCvtStringToRenderTable(Display *display,
+		       XrmValue *args,
+		       Cardinal *num_args,
+		       XrmValue *from,
+		       XrmValue *to,
+		       XtPointer *converter_data)
+{
+	return False;
 }
 #endif
