@@ -1,6 +1,6 @@
 /*
  *
- * $Id: ScrolledW.c,v 1.117 1999/09/02 23:28:39 rwscott Exp $
+ * $Id: ScrolledW.c,v 1.118 2000/01/09 03:18:16 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ScrolledW.c,v 1.117 1999/09/02 23:28:39 rwscott Exp $";
+static const char rcsid[] = "$Id: ScrolledW.c,v 1.118 2000/01/09 03:18:16 rwscott Exp $";
 
 /*
  * Current status of changes in layout (Danny 18/4/1997) :
@@ -1307,11 +1307,13 @@ change_managed(Widget w)
     {
 	_XmScrolledWPreferredSize(w, NULL, NULL, &vals);
 
-	_XmScrolledWGeomRequest(w, &vals);
+	if (_XmScrolledWGeomRequest(w, &vals) == XtGeometryYes)
+	{
 
-	_XmScrolledWLayout(w, NULL, NULL, &vals);
+	    _XmScrolledWLayout(w, NULL, NULL, &vals);
 
-	_XmScrolledWConfigureChildren(w, NULL, NULL, &vals);
+	    _XmScrolledWConfigureChildren(w, NULL, NULL, &vals);
+	}
 
 	DEBUGOUT(XdbDebug(__FILE__, w, "change_managed: size %d %d\n",
 			  XtWidth(w), XtHeight(w)));
