@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ToggleB.c,v 1.68 1999/07/27 05:13:49 gritton Exp $
+ * $Id: ToggleB.c,v 1.70 1999/11/11 13:43:25 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ToggleB.c,v 1.68 1999/07/27 05:13:49 gritton Exp $";
+static const char rcsid[] = "$Id: ToggleB.c,v 1.70 1999/11/11 13:43:25 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -38,10 +38,10 @@ static const char rcsid[] = "$Id: ToggleB.c,v 1.68 1999/07/27 05:13:49 gritton E
 #include <Xm/TransltnsP.h>
 #include <X11/ShellP.h>
 
-#ifndef __STDC__
-#include <varargs.h>
-#else
+#if STDC_HEADERS
 #include <stdarg.h>
+#else
+#include <varargs.h>
 #endif
 
 #include <XmI/DebugUtil.h>
@@ -603,19 +603,49 @@ initialize(Widget request, Widget new_w,
 
     if (TB_IndOn(new_w))
     {
-	int iheight;
+	int margin;
 
-	/* Make room for the indicator */
-	iheight = TB_IndicatorDim(new_w);
+	/* Make sure there's enough room on the side for the indicator */
+	margin = (TB_IndicatorDim(new_w) + TB_Spacing(new_w))
+	    - (Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+	       ? Lab_MarginLeft(new_w) : Lab_MarginRight(new_w));
+	if (margin > 0)
+	{
+	    if (Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R)
+	    {
+		Lab_MarginLeft(new_w) += margin;
+		Lab_TextRect_x(new_w) += margin;
+		Lab_AccTextRect(new_w).x += margin;
+	    }
+	    else
+		Lab_MarginRight(new_w) += margin;
+	    if (!XtWidth(request))
+		XtWidth(new_w) += margin;
+	}
+
+	/* Make sure there's enough room vertically.
+	 * Non-menu toggles want some padding space.
+	 */
+	margin = TB_IndicatorDim(new_w) - (Lab_TextRect_height(new_w)
+					   + Lab_MarginTop(new_w)
+					   + Lab_MarginBottom(new_w));
 #ifdef LESS_VERTICAL_PADDING
-	iheight -= Lab_MarginHeight(new_w) << 1;
+	margin -= Lab_MarginHeight(new_w) << 1;
 #else
 	if (!IN_MENU(new_w))
-	    iheight += (Lab_Shadow(new_w) + Xm3D_ENHANCE_PIXEL) << 1;
+	    margin += (Lab_Shadow(new_w) + Xm3D_ENHANCE_PIXEL) << 1;
 #endif
-	_LtLabelAccomodateIndicator(new_w, request,
-	    TB_IndicatorDim(new_w) + TB_Spacing(new_w), iheight,
-	    0, Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R, False);
+	if (margin > 0)
+	{
+	    Lab_MarginTop(new_w) += margin >> 1;
+	    Lab_MarginBottom(new_w) += margin >> 1;
+	    if (!XtHeight(request))
+	    {
+		Lab_TextRect_y(new_w) += margin >> 1;
+		Lab_AccTextRect(new_w).y += margin >> 1;
+		XtHeight(new_w) += margin;
+	    }
+	}
     }
 
     if (IN_MENU(new_w))
@@ -765,7 +795,10 @@ set_values(Widget old, Widget request, Widget new_w,
     if (Lab_IsPixmap(new_w))
 	TB_IndicatorSet(new_w) = True;
 
-    /* Adjust margins for the indicator size if necessary */
+    /* Adjust margins for the indicator size if necessary.
+     * Margins be increased or decreased; if the margin was explicitly set
+     * in this call, don't decrease it past that (though it can get bigger).
+     */
     if (TB_IndOn(new_w) &&
 	(TB_IndicatorDim(new_w) != TB_IndicatorDim(old)
 	 || TB_Spacing(new_w) != TB_Spacing(old)
@@ -777,18 +810,67 @@ set_values(Widget old, Widget request, Widget new_w,
 	 || Lab_MarginTop(new_w) != Lab_MarginTop(old)
 	 || Lab_MarginBottom(new_w) != Lab_MarginBottom(old)))
     {
-	int iheight;
+	int margin, tm;
 
-	iheight = TB_IndicatorDim(new_w);
+	margin = TB_IndicatorDim(new_w) + TB_Spacing(new_w)
+	    - (Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+	       ? Lab_MarginLeft(new_w) : Lab_MarginRight(new_w));
+	if (margin && (margin > 0 ||
+		       (Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+			? Lab_MarginLeft(new_w) == Lab_MarginLeft(old)
+			: Lab_MarginRight(new_w) == Lab_MarginRight(old))))
+	{
+	    if (Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R)
+	    {
+		Lab_MarginLeft(new_w) += margin;
+		Lab_TextRect_x(new_w) += margin;
+		Lab_AccTextRect(new_w).x += margin;
+	    }
+	    else
+		Lab_MarginRight(new_w) += margin;
+	    if (Lab_RecomputeSize(new_w) || !XtWidth(request))
+		XtWidth(new_w) += margin;
+	}
+
+	margin = TB_IndicatorDim(new_w) - (Lab_TextRect_height(new_w)
+					   + Lab_MarginTop(new_w)
+					   + Lab_MarginBottom(new_w));
 #ifdef LESS_VERTICAL_PADDING
-	iheight -= Lab_MarginHeight(new_w) << 1;
+	margin -= Lab_MarginHeight(new_w) << 1;
 #else
 	if (!IN_MENU(new_w))
-	    iheight += (Lab_Shadow(new_w) + Xm3D_ENHANCE_PIXEL) << 1;
+	    margin += (Lab_Shadow(new_w) + Xm3D_ENHANCE_PIXEL) << 1;
 #endif
-	_LtLabelAccomodateIndicator(new_w, request,
-	    TB_IndicatorDim(new_w) + TB_Spacing(new_w), iheight,
-	    0, Lab_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R, True);
+	if (margin)
+	{
+	    margin /= 2;
+	    tm = margin;
+	    if (tm < (Lab_MarginTop(new_w) == Lab_MarginTop(old)
+		      ? XmDEFAULT_TOP_MARGIN - (int)Lab_MarginTop(new_w)
+		      : 0))
+		tm = (Lab_MarginTop(new_w) == Lab_MarginTop(old)
+		      ? XmDEFAULT_TOP_MARGIN - (int)Lab_MarginTop(new_w)
+		      : 0);
+	    Lab_MarginTop(new_w) += tm;
+	    if (Lab_RecomputeSize(new_w) || !XtHeight(request))
+		XtHeight(new_w) += tm;
+
+	    if (margin < (Lab_MarginBottom(new_w) == Lab_MarginBottom(old)
+		       ? XmDEFAULT_BOTTOM_MARGIN - (int)Lab_MarginBottom(new_w)
+		       : 0))
+		margin = (Lab_MarginBottom(new_w) == Lab_MarginBottom(old)
+		       ? XmDEFAULT_BOTTOM_MARGIN - (int)Lab_MarginBottom(new_w)
+		       : 0);
+	    Lab_MarginBottom(new_w) += margin;
+	    if (Lab_RecomputeSize(new_w) || !XtHeight(request))
+		XtHeight(new_w) += margin;
+
+	    if (tm != margin)
+	    {
+		Lab_TextRect_y(new_w) += (tm - margin) / 2;
+		Lab_AccTextRect(new_w).y += (tm - margin) / 2;
+	    }
+	}
 
 	refresh_needed = True;
     }
@@ -1183,7 +1265,7 @@ KeySelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /* FIX ME */
 }
 
-#ifdef __STDC__
+#if STDC_HEADERS
 static void
 MenuProcEntry(int proc, Widget w,...)
 {
