@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Vendor.c,v 1.47 1999/07/25 11:53:12 rwscott Exp $
+ * $Id: Vendor.c,v 1.50 2000/01/04 03:32:31 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Vendor.c,v 1.47 1999/07/25 11:53:12 rwscott Exp $";
+static const char rcsid[] = "$Id: Vendor.c,v 1.50 2000/01/04 03:32:31 rwscott Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -1009,6 +1009,7 @@ destroy(Widget w)
     XmWidgetExtData data;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "VendorShell Destroy()\n"));
+    DEBUGOUT(XdbDebug("NEDIT", w, "VendorShell Destroy()\n"));
 
     _XmPopWidgetExtData(w, &data, XmSHELL_EXTENSION);
     if (data != NULL) {
@@ -1406,6 +1407,18 @@ LTAddGrab(Widget wid, Boolean exclusive, Boolean spring_loaded,
     }
 
     d = XmGetXmDisplay(XtDisplayOfObject(wid));
+    {
+    int i;
+    XmModalData ps = Display_Modals(d);
+
+	for (i = Display_NumModals(d); i > 0; i--, ps++)
+	{
+	    if (ps->wid == wid)
+	    {
+		return;
+	    }
+	}
+    }
     dump_grab_list(d);
 
     /* grow the modal list, if necessary */
@@ -1518,7 +1531,7 @@ LTRemoveGrab(Widget wid, XmVendorShellExtObject ve,
 
     for (i = Display_NumModals(d); i > 0; i--, ps++)
     {
-	if ((ps->wid == wid) && remove_grab_physically)
+	if ((ps->wid == wid) && !CoreBeingDestroyed(wid) && remove_grab_physically)
 	{
 	    XtRemoveGrab(wid);
 	}
@@ -1604,7 +1617,15 @@ LTRemoveGrabCallback(Widget wid, XtPointer client_data,
     if (!XtIsSubclass(wid, vendorShellWidgetClass))
 #endif
     {
+#if 0
+	/* rws 18 Sep 1999
+	   This seems to be the problem of the nedit File->Open problem
+	   Also ddd --separate-windows
+	 */
 	LTRemoveGrab(wid, (XmVendorShellExtObject)client_data, False);
+#else
+	LTRemoveGrab(wid, (XmVendorShellExtObject)client_data, True);
+#endif
     }
 }
 
@@ -3175,7 +3196,7 @@ _LtFindVendorExt(Widget w)
    done by the _DLL_InitTerm-function under OS/2.
  */
 #ifdef __EMX__
-void
+static void
 _XmFixupVendorShell()
 {
     transientShellWidgetClass->core_class.superclass =
@@ -3187,7 +3208,7 @@ _XmFixupVendorShell()
 unsigned long
 _DLL_InitTerm(unsigned long mod_handle, unsigned long flag)
 {
-    if (flag == 0)
+    if (flag == 0)  /* initialize DLL */
     {
 	_XmFixupVendorShell();
     }
