@@ -15,6 +15,7 @@
 #include <Xm/SelectioB.h>
 #include <Xm/DrawingA.h>
 #include <Xm/Label.h>
+#include <X11/keysym.h>
 #include <stdio.h>
 #include "../../common/Test.h"
 
@@ -237,6 +238,46 @@ _CallbackResult Results[] = { /* insert results of Motif run after this */
 	"pane3",                XmCR_UNMAP,\
 	"button4",              XmCR_ACTIVATE,\
 	"button4",              XmCR_DISARM,\
+
+	"cascade1",             XmCR_CASCADING,\
+	"pane1",                XmCR_MAP,\
+	"pane1",                XmCR_UNMAP,\
+	"pane1",                XmCR_TEAR_OFF_ACTIVATE,\
+	"pane1",                XmCR_MAP,\
+	"button1",              XmCR_ARM,\
+	"button1",              XmCR_DISARM,\
+	"pane1",                XmCR_UNMAP,\
+	"pane1",                XmCR_TEAR_OFF_DEACTIVATE,\
+	"cascade1",             XmCR_CASCADING,\
+	"pane1",                XmCR_MAP,\
+	"pane1",                XmCR_UNMAP,\
+	"pane1",                XmCR_TEAR_OFF_ACTIVATE,\
+	"pane1",                XmCR_MAP,\
+	"button1",              XmCR_ARM,\
+	"cascade2",             XmCR_CASCADING,\
+	"pane2",                XmCR_MAP,\
+	"button1",              XmCR_DISARM,\
+	"pane2",                XmCR_UNMAP,\
+	"pane2",                XmCR_TEAR_OFF_ACTIVATE,\
+	"pane2",                XmCR_MAP,\
+	"button2",              XmCR_ARM,\
+	"button2",              XmCR_ACTIVATE,\
+	"button2",              XmCR_DISARM,\
+	"button2",              XmCR_ARM,\
+        "button2",              XmCR_DISARM,\
+        "pane2",                XmCR_UNMAP,\
+        "pane2",                XmCR_TEAR_OFF_DEACTIVATE,\
+        "pane1",                XmCR_UNMAP,\
+        "pane1",                XmCR_TEAR_OFF_DEACTIVATE,\
+        "cascade1",		XmCR_CASCADING, \
+        "pane1",		XmCR_MAP, \
+        "button1",		XmCR_ARM, \
+        "button1",		XmCR_DISARM, \
+        "pane1",		XmCR_UNMAP, \
+        "button3",		XmCR_ARM, \
+        "button3",		XmCR_ACTIVATE, \
+        "button3",		XmCR_DISARM, \
+
 	NULL, 0,  /* This must stay here */
 };
 int ResultIndex = 0;
@@ -738,6 +779,33 @@ main(int argc, char **argv)
     LessTifTestBtn1Up(button4);
     printf("\n");
 
+    LessTifTestBtn1Down(cascade1);
+    LessTifTestBtn1Up(pane1_tear_off);
+    LessTifTestWaitForIt(XtParent(pane1));
+    LessTifTestBtn1Down(cascade1);
+    LessTifTestBtn1Up(pane1_tear_off);
+    LessTifTestWaitForIt(XtParent(pane1));
+    LessTifTestBtn1Down(cascade2);
+    LessTifTestBtn1Up(pane2_tear_off);
+    LessTifTestWaitForIt(XtParent(pane2));
+    LessTifTestBtn1Down(button2);
+    LessTifTestBtn1Up(button2);
+
+    LessTifTestEsc(button2);
+    LessTifTestEsc(button1);
+
+    XtVaSetValues(button3,
+    	XmNmnemonic, 'b',
+    	NULL);
+    LessTifTestKeyPress(cascade1, XK_Alt_L, 0x0);
+    LessTifTestKeyPress(cascade1, XK_C, 0x8);
+    LessTifTestKeyRelease(XtParent(cascade1), XK_C, 0x8);
+    LessTifTestKeyRelease(XtParent(cascade1), XK_Alt_L, 0x8);
+    LessTifTestWaitForIt(XtParent(pane1));
+    LessTifTestKeyRelease(XtParent(cascade1), XK_B, 0x0);
+
+    printf("\n");
+
 #if 0
     /*
     LessTifTestPrintEvents(toplevel, False);
