@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Test.c,v 1.19 1999/04/13 16:49:21 jon Exp $
+ * $Id: Test.c,v 1.23 1999/11/13 03:01:08 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,10 +22,12 @@
  *
  **/
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <X11/X.h>
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
+#include <X11/keysym.h>
 #include <X11/Core.h>
 
 /* ************** */
@@ -577,6 +579,164 @@ LessTifTestWarpPointer(Widget w)
 
 /* ************** */
 
+void
+LessTifTestKeyRelease(Widget w, KeySym keysym, unsigned int state)
+{
+    Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
+    Window window = XtIsSubclass(w, coreWidgetClass) ? XtWindow(w) : XtWindow(XtParent(w));
+    XKeyEvent key_event =
+    {
+	KeyRelease,
+	0,
+	False,
+	NULL, /* display, */
+	None, /* window, */
+	None, /* root, */
+	None, /* subwindow, */
+	CurrentTime,
+	0, 0,
+	0, 0,
+	0,
+	0x09,
+	True
+    };
+    XEvent *event;
+    unsigned int keys_buttons;
+
+    key_event.keycode = XKeysymToKeycode(display, keysym);
+    key_event.display=display;
+    key_event.state=state;
+    key_event.window=key_event.root=key_event.subwindow=window;
+    LessTifTestWarpPointer(w);
+    XQueryPointer(display, window,
+		  &key_event.root,
+		  &key_event.subwindow,
+		  &key_event.x_root, &key_event.y_root,
+		  &key_event.x, &key_event.y,
+		  &keys_buttons);
+    event = (XEvent*)&key_event;
+    event->xany.serial = NextRequest(display);
+    event->xbutton.time = SynthTime;
+    SynthTime += 200;
+    XSendEvent(display, key_event.window /*PointerWindow*/, True, KeyReleaseMask, event);
+    /*
+    key_event.type = KeyRelease;
+    key_event.window = XtWindow(XtParent(w));
+    XSendEvent(display, key_event.window, True, KeyReleaseMask, event);
+    */
+    LessTifTestFlushEvents(w);
+    Delay(w);
+}
+
+/* ************** */
+
+void
+LessTifTestKeyPress(Widget w, KeySym keysym, unsigned int state)
+{
+    Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
+    Window window = XtIsSubclass(w, coreWidgetClass) ? XtWindow(w) : XtWindow(XtParent(w));
+    XKeyEvent key_event =
+    {
+	KeyPress,
+	0,
+	False,
+	NULL, /* display, */
+	None, /* window, */
+	None, /* root, */
+	None, /* subwindow, */
+	CurrentTime,
+	0, 0,
+	0, 0,
+	0,
+	0x09,
+	True
+    };
+    XEvent *event;
+    unsigned int keys_buttons;
+
+    key_event.keycode = XKeysymToKeycode(display, keysym);
+    key_event.display=display;
+    key_event.state=state;
+    key_event.window=key_event.root=key_event.subwindow=window;
+    LessTifTestWarpPointer(w);
+    XQueryPointer(display, window,
+		  &key_event.root,
+		  &key_event.subwindow,
+		  &key_event.x_root, &key_event.y_root,
+		  &key_event.x, &key_event.y,
+		  &keys_buttons);
+    event = (XEvent*)&key_event;
+    event->xany.serial = NextRequest(display);
+    event->xbutton.time = SynthTime;
+    SynthTime += 200;
+    XSendEvent(display, InputFocus /*PointerWindow*/, True, KeyPressMask, event);
+    /*
+    key_event.type = KeyRelease;
+    key_event.window = XtWindow(XtParent(w));
+    XSendEvent(display, key_event.window, True, KeyReleaseMask, event);
+    */
+    LessTifTestFlushEvents(w);
+    Delay(w);
+}
+
+/* ************** */
+
+void
+LessTifTestKeysym(Widget w, KeySym keysym)
+{
+    Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
+    Window window = XtIsSubclass(w, coreWidgetClass) ? XtWindow(w) : XtWindow(XtParent(w));
+    XKeyEvent key_event =
+    {
+	KeyPress,
+	0,
+	False,
+	NULL, /* display, */
+	None, /* window, */
+	None, /* root, */
+	None, /* subwindow, */
+	CurrentTime,
+	0, 0,
+	0, 0,
+	0,
+	0x09,
+	True
+    };
+    XEvent *event;
+    unsigned int keys_buttons;
+
+    key_event.keycode = XKeysymToKeycode(display, keysym);
+    key_event.display=display;
+    key_event.window=key_event.root=key_event.subwindow=window;
+    LessTifTestWarpPointer(w);
+    XQueryPointer(display, window,
+		  &key_event.root,
+		  &key_event.subwindow,
+		  &key_event.x_root, &key_event.y_root,
+		  &key_event.x, &key_event.y,
+		  &keys_buttons);
+    event = (XEvent*)&key_event;
+    event->xany.serial = NextRequest(display);
+    event->xbutton.time = SynthTime;
+    SynthTime += 200;
+    XSendEvent(display, InputFocus /*PointerWindow*/, True, KeyPressMask, event);
+    key_event.type = KeyRelease;
+    key_event.window = XtWindow(XtParent(w));
+    XSendEvent(display, key_event.window /*PointerWindow*/, True, KeyReleaseMask, event);
+    LessTifTestFlushEvents(w);
+    Delay(w);
+}
+
+/* ************** */
+
+void
+LessTifTestEsc(Widget w)
+{
+    LessTifTestKeysym(w, XK_Escape);
+}
+
+/* ************** */
+
 void
 LessTifTestBtn1Down(Widget w)
 {
